@@ -1,15 +1,32 @@
-#! /usr/bin/python
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 from twisted.internet import gtk2reactor
 gtk2reactor.install()
+import gtk.glade  # @UnresolvedImport
+import re
+
+from twisted.cred import credentials
 from twisted.internet import reactor
 from twisted.python import util
 from twisted.spread import pb
-from twisted.cred import credentials
-import gtk, gtk.glade, gnome.ui
-import sys, re
+
 
 class DebugWidget:
+
     def __init__(self, master="localhost:8007", passwd="debugpw"):
         self.connected = 0
         try:
@@ -60,6 +77,7 @@ class DebugWidget:
             d = f.login(creds)
             reactor.connectTCP(self.host, int(self.port), f)
             d.addCallbacks(self.connect_complete, self.connect_failed)
+
     def connect_complete(self, ref):
         self.connectbutton.set_label("Disconnect")
         self.connectlabel.set_text("Connected")
@@ -67,9 +85,11 @@ class DebugWidget:
         self.remote = ref
         self.remote.callRemote("print", "hello cleveland")
         self.remote.notifyOnDisconnect(self.disconnected)
+
     def connect_failed(self, why):
         self.connectlabel.set_text("Failed")
         print why
+
     def disconnected(self, ref):
         self.connectbutton.set_label("Connect")
         self.connectlabel.set_text("Disconnected")
@@ -81,9 +101,11 @@ class DebugWidget:
             return
         d = self.remote.callRemote("reload")
         d.addErrback(self.err)
+
     def do_rebuild(self, widget):
         print "Not yet implemented"
         return
+
     def do_poke_irc(self, widget):
         if not self.remote:
             return
@@ -105,8 +127,9 @@ class DebugWidget:
             if revision == '':
                 revision = None
         reason = "debugclient 'Request Build' button pushed"
+        properties = {}
         d = self.remote.callRemote("requestBuild",
-                                   name, reason, branch, revision)
+                                   name, reason, branch, revision, properties)
         d.addErrback(self.err)
 
     def do_ping(self, widget):
@@ -152,7 +175,7 @@ class DebugWidget:
             if revision == '':
                 revision = None
 
-        kwargs = { 'revision': revision, 'who': who }
+        kwargs = {'revision': revision, 'who': who}
         if branch:
             kwargs['branch'] = branch
         d = self.remote.callRemote("fakeChange", filename, **kwargs)
@@ -164,20 +187,9 @@ class DebugWidget:
         name = self.buildname.get_text()
         d = self.remote.callRemote("setCurrentState", name, state)
         d.addErrback(self.err)
+
     def err(self, failure):
-        print "received error"
-        failure.printTraceback()
-        
+        print "received error:", failure
 
     def run(self):
         reactor.run()
-
-if __name__ == '__main__':
-    master = "localhost:8007"
-    if len(sys.argv) > 1:
-        master = sys.argv[1]
-    passwd = "debugpw"
-    if len(sys.argv) > 2:
-        passwd = sys.argv[2]
-    d = DebugWidget(master, passwd)
-    d.run()
