@@ -18,26 +18,30 @@ Support for buildsets in the database
 """
 
 import sqlalchemy as sa
-from twisted.internet import reactor
-from buildbot.util import json
+
 from buildbot.db import base
-from buildbot.util import epoch2datetime, datetime2epoch
+from buildbot.util import datetime2epoch
+from buildbot.util import epoch2datetime
+from buildbot.util import json
+from twisted.internet import reactor
+
 
 class BsDict(dict):
     pass
 
+
 class BuildsetsConnectorComponent(base.DBConnectorComponent):
     # Documentation is in developer/database.rst
 
     def addBuildset(self, sourcestampsetid, reason, properties, builderNames,
-                   external_idstring=None, _reactor=reactor):
+                    external_idstring=None, _reactor=reactor):
         def thd(conn):
             buildsets_tbl = self.db.model.buildsets
             submitted_at = _reactor.seconds()
 
             self.check_length(buildsets_tbl.c.reason, reason)
             self.check_length(buildsets_tbl.c.external_idstring,
-                    external_idstring)
+                              external_idstring)
 
             transaction = conn.begin()
 
@@ -54,13 +58,11 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
 
                 inserts = [
                     dict(buildsetid=bsid, property_name=k,
-                         property_value=json.dumps([v,s]))
-                    for k,(v,s) in properties.iteritems() ]
+                         property_value=json.dumps([v, s]))
+                    for k, (v, s) in properties.iteritems()]
                 for i in inserts:
                     self.check_length(bs_props_tbl.c.property_name,
                                       i['property_name'])
-                    self.check_length(bs_props_tbl.c.property_value,
-                                      i['property_value'])
 
                 conn.execute(bs_props_tbl.insert(), inserts)
 
@@ -74,10 +76,10 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
             for buildername in builderNames:
                 self.check_length(br_tbl.c.buildername, buildername)
                 r = conn.execute(ins,
-                    dict(buildsetid=bsid, buildername=buildername, priority=0,
-                        claimed_at=0, claimed_by_name=None,
-                        claimed_by_incarnation=None, complete=0, results=-1,
-                        submitted_at=submitted_at, complete_at=None))
+                                 dict(buildsetid=bsid, buildername=buildername, priority=0,
+                                      claimed_at=0, claimed_by_name=None,
+                                      claimed_by_incarnation=None, complete=0, results=-1,
+                                      submitted_at=submitted_at, complete_at=None))
 
                 brids[buildername] = r.inserted_primary_key[0]
 
@@ -87,7 +89,7 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
         return self.db.pool.do(thd)
 
     def completeBuildset(self, bsid, results, complete_at=None,
-                                _reactor=reactor):
+                         _reactor=reactor):
         if complete_at is not None:
             complete_at = datetime2epoch(complete_at)
         else:
@@ -100,9 +102,9 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
                 (tbl.c.id == bsid) &
                 ((tbl.c.complete == None) | (tbl.c.complete != 1))))
             res = conn.execute(q,
-                complete=1,
-                results=results,
-                complete_at=complete_at)
+                               complete=1,
+                               results=results,
+                               complete_at=complete_at)
 
             if res.rowcount != 1:
                 raise KeyError
@@ -130,7 +132,7 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
                     q = q.where((bs_tbl.c.complete == 0) |
                                 (bs_tbl.c.complete == None))
             res = conn.execute(q)
-            return [ self._row2dict(row) for row in res.fetchall() ]
+            return [self._row2dict(row) for row in res.fetchall()]
         return self.db.pool.do(thd)
 
     def getRecentBuildsets(self, count, branch=None, repository=None,
@@ -139,10 +141,10 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
             bs_tbl = self.db.model.buildsets
             ss_tbl = self.db.model.sourcestamps
             j = sa.join(self.db.model.buildsets,
-                               self.db.model.sourcestampsets)
+                        self.db.model.sourcestampsets)
             j = j.join(self.db.model.sourcestamps)
             q = sa.select(columns=[bs_tbl], from_obj=[j],
-                                         distinct=True)
+                          distinct=True)
             q = q.order_by(sa.desc(bs_tbl.c.submitted_at))
             q = q.limit(count)
 
@@ -153,12 +155,12 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
                     q = q.where((bs_tbl.c.complete == 0) |
                                 (bs_tbl.c.complete == None))
             if branch:
-              q = q.where(ss_tbl.c.branch == branch)
+                q = q.where(ss_tbl.c.branch == branch)
             if repository:
-              q = q.where(ss_tbl.c.repository == repository)
+                q = q.where(ss_tbl.c.repository == repository)
             res = conn.execute(q)
-            return list(reversed([ self._row2dict(row)
-                                  for row in res.fetchall() ]))
+            return list(reversed([self._row2dict(row)
+                                  for row in res.fetchall()]))
         return self.db.pool.do(thd)
 
     def getBuildsetProperties(self, buildsetid):
@@ -177,14 +179,14 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
         def thd(conn):
             bsp_tbl = self.db.model.buildset_properties
             q = sa.select(
-                [ bsp_tbl.c.property_name, bsp_tbl.c.property_value ],
+                [bsp_tbl.c.property_name, bsp_tbl.c.property_value],
                 whereclause=(bsp_tbl.c.buildsetid == buildsetid))
             l = []
             for row in conn.execute(q):
                 try:
                     properties = json.loads(row.property_value)
                     l.append((row.property_name,
-                           tuple(properties)))
+                              tuple(properties)))
                 except ValueError:
                     pass
             return dict(l)
@@ -195,8 +197,8 @@ class BuildsetsConnectorComponent(base.DBConnectorComponent):
             if epoch:
                 return epoch2datetime(epoch)
         return BsDict(external_idstring=row.external_idstring,
-                reason=row.reason, sourcestampsetid=row.sourcestampsetid,
-                submitted_at=mkdt(row.submitted_at),
-                complete=bool(row.complete),
-                complete_at=mkdt(row.complete_at), results=row.results,
-                bsid=row.id)
+                      reason=row.reason, sourcestampsetid=row.sourcestampsetid,
+                      submitted_at=mkdt(row.submitted_at),
+                      complete=bool(row.complete),
+                      complete_at=mkdt(row.complete_at), results=row.results,
+                      bsid=row.id)
