@@ -16,21 +16,23 @@
 
 import re
 import types
-from email.Message import Message
-from email.Utils import formatdate
-from email.MIMEText import MIMEText
-from email.MIMENonMultipart import MIMENonMultipart
-from email.MIMEMultipart import MIMEMultipart
-from StringIO import StringIO
 import urllib
 
-from zope.interface import implements
-from twisted.internet import defer, reactor
+from StringIO import StringIO
+from email.message import Message
+from email.mime.multipart import MIMEMultipart
+from email.mime.nonmultipart import MIMENonMultipart
+from email.mime.text import MIMEText
+from email.utils import formatdate
+
+from twisted.internet import defer
+from twisted.internet import reactor
 from twisted.python import log as twlog
+from zope.interface import implements
 
 try:
     from twisted.mail.smtp import ESMTPSenderFactory
-    ESMTPSenderFactory = ESMTPSenderFactory # for pyflakes
+    ESMTPSenderFactory = ESMTPSenderFactory  # for pyflakes
 except ImportError:
     ESMTPSenderFactory = None
 
@@ -43,19 +45,39 @@ except ImportError:
 
 # this incantation teaches email to output utf-8 using 7- or 8-bit encoding,
 # although it has no effect before python-2.7.
-from email import Charset
-Charset.add_charset('utf-8', Charset.SHORTEST, None, 'utf-8')
+from email import charset
+charset.add_charset('utf-8', charset.SHORTEST, None, 'utf-8')
 
-from buildbot import interfaces, util, config
+from buildbot import config
+from buildbot import interfaces
+from buildbot import util
 from buildbot.process.users import users
 from buildbot.status import base
-from buildbot.status.results import FAILURE, SUCCESS, WARNINGS, EXCEPTION, Results
+from buildbot.status import buildset
+from buildbot.status.results import EXCEPTION
+from buildbot.status.results import FAILURE
+from buildbot.status.results import Results
+from buildbot.status.results import SUCCESS
+from buildbot.status.results import WARNINGS
 
-VALID_EMAIL = re.compile("[a-zA-Z0-9\.\_\%\-\+]+@[a-zA-Z0-9\.\_\%\-]+.[a-zA-Z]{2,6}")
+# Email parsing can be complex. We try to take a very liberal
+# approach. The local part of an email address matches ANY non
+# whitespace character. Rather allow a malformed email address than
+# croaking on a valid (the matching of domains should be correct
+# though; requiring the domain to not be a top level domain). With
+# these regular expressions, we can match the following:
+#
+#    full.name@example.net
+#    Full Name <full.name@example.net>
+#    <full.name@example.net>
+VALID_EMAIL_ADDR = r"(?:\S+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)+\.?)"
+VALID_EMAIL = re.compile(r"^(?:%s|(.+\s+)?<%s>\s*)$" % ((VALID_EMAIL_ADDR,) * 2))
+VALID_EMAIL_ADDR = re.compile(VALID_EMAIL_ADDR)
 
 ENCODING = 'utf8'
 LOG_ENCODING = 'utf-8'
 
+
 class Domain(util.ComparableMixin):
     implements(interfaces.IEmailLookup)
     compare_attrs = ["domain"]
@@ -71,69 +93,86 @@ class Domain(util.ComparableMixin):
         return name + "@" + self.domain
 
 
-def defaultMessage(mode, name, build, results, master_status):
-    """Generate a buildbot mail message and return a tuple of message text
-        and type."""
-    ss_list = build.getSourceStamps()
-    
+def _defaultMessageIntro(mode, results, build):
     prev = build.getPreviousBuild()
 
-    text = ""
     if results == FAILURE:
         if "change" in mode and prev and prev.getResults() != results or \
-               "problem" in mode and prev and prev.getResults() != FAILURE:
-            text += "The Buildbot has detected a new failure"
+                "problem" in mode and prev and prev.getResults() != FAILURE:
+            text = "The Buildbot has detected a new failure"
         else:
-            text += "The Buildbot has detected a failed build"
+            text = "The Buildbot has detected a failed build"
     elif results == WARNINGS:
-        text += "The Buildbot has detected a problem in the build"
+        text = "The Buildbot has detected a problem in the build"
     elif results == SUCCESS:
         if "change" in mode and prev and prev.getResults() != results:
-            text += "The Buildbot has detected a restored build"
+            text = "The Buildbot has detected a restored build"
         else:
-            text += "The Buildbot has detected a passing build"
+            text = "The Buildbot has detected a passing build"
     elif results == EXCEPTION:
-        text += "The Buildbot has detected a build exception"
+        text = "The Buildbot has detected a build exception"
 
+    return text
+
+
+def _defaultMessageProjects(source_stamps, master_status):
     projects = []
-    if ss_list:
-        for ss in ss_list:
-            if ss.project and ss.project not in projects:
-                projects.append(ss.project)
+
+    for ss in source_stamps:
+        if ss.project and ss.project not in projects:
+            projects.append(ss.project)
+
     if not projects:
         projects = [master_status.getTitle()]
-    text += " on builder %s while building %s.\n" % (name, ', '.join(projects))
 
-    if master_status.getURLForThing(build):
-        text += "Full details are available at:\n %s\n" % master_status.getURLForThing(build)
+    return ', '.join(projects)
+
+
+def _defaultMessageURLs(master_status, build):
+    text = ""
+
+    # add build URL if available
+    url = master_status.getURLForThing(build)
+    if url:
+        text += " Full details are available at:\n    %s\n" % url
+
     text += "\n"
 
-    if master_status.getBuildbotURL():
-        text += "Buildbot URL: %s\n\n" % urllib.quote(master_status.getBuildbotURL(), '/:')
+    # add buildbot main URL if available
+    url = master_status.getBuildbotURL()
+    if url:
+        text += "Buildbot URL: %s\n\n" % urllib.quote(url, '/:')
 
-    text += "Buildslave for this Build: %s\n\n" % build.getSlavename()
-    text += "Build Reason: %s\n" % build.getReason()
+    return text
 
-    for ss in ss_list:
+
+def _defaultMessageSourceStamps(source_stamps):
+    text = ""
+
+    for ss in source_stamps:
         source = ""
-        if ss and ss.branch:
+
+        if ss.branch:
             source += "[branch %s] " % ss.branch
-        if ss and ss.revision:
+
+        if ss.revision:
             source += str(ss.revision)
         else:
             source += "HEAD"
-        if ss and ss.patch:
+
+        if ss.patch:
             source += " (plus patch)"
 
         discriminator = ""
         if ss.codebase:
             discriminator = " '%s'" % ss.codebase
+
         text += "Build Source Stamp%s: %s\n" % (discriminator, source)
 
-    text += "Blamelist: %s\n" % ",".join(build.getResponsibleUsers())
+    return text
 
-    text += "\n"
 
+def _defaultMessageSummary(build, results):
     t = build.getText()
     if t:
         t = ": " + " ".join(t)
@@ -141,22 +180,47 @@ def defaultMessage(mode, name, build, results, master_status):
         t = ""
 
     if results == SUCCESS:
-        text += "Build succeeded!\n"
+        text = "Build succeeded!\n"
     elif results == WARNINGS:
-        text += "Build Had Warnings%s\n" % t
+        text = "Build Had Warnings%s\n" % t
     else:
-        text += "BUILD FAILED%s\n" % t
+        text = "BUILD FAILED%s\n" % t
 
+    return text
+
+
+def defaultMessage(mode, name, build, results, master_status):
+    """Generate a buildbot mail message and return a tuple of message text
+        and type."""
+    ss_list = build.getSourceStamps()
+
+    text = _defaultMessageIntro(mode, results, build)
+    text += " on builder %s while building %s." % \
+            (name, _defaultMessageProjects(ss_list, master_status))
+
+    text += _defaultMessageURLs(master_status, build)
+
+    text += "Buildslave for this Build: %s\n\n" % build.getSlavename()
+    text += "Build Reason: %s\n" % build.getReason()
+    text += _defaultMessageSourceStamps(ss_list)
+    text += "Blamelist: %s\n" % ",".join(build.getResponsibleUsers())
     text += "\n"
-    text += "sincerely,\n"
+
+    text += _defaultMessageSummary(build, results)
+    text += "\n"
+    text += "Sincerely,\n"
     text += " -The Buildbot\n"
     text += "\n"
-    return { 'body' : text, 'type' : 'plain' }
+
+    return {'body': text, 'type': 'plain'}
+
 
 def defaultGetPreviousBuild(current_build):
         return current_build.getPreviousBuild()
 
-class MailNotifier(base.StatusReceiverMultiService):
+
+class MailNotifier(base.StatusReceiverMultiService, buildset.BuildSetSummaryNotifierMixin):
+
     """This is a status notifier which sends email to a list of recipients
     upon the completion of each build. It can be configured to only send out
     mail for certain builds, and only send messages when the build fails, or
@@ -254,12 +318,12 @@ class MailNotifier(base.StatusReceiverMultiService):
         @type  relayhost: string
         @param relayhost: the host to which the outbound SMTP connection
                           should be made. Defaults to 'localhost'
-                          
+
         @type  buildSetSummary: boolean
         @param buildSetSummary: if True, this notifier will only send a summary
                                 email when a buildset containing any of its
                                 watched builds completes
-                                
+
         @type  lookup:    implementor of {IEmailLookup}
         @param lookup:    object which provides IEmailLookup, which is
                           responsible for mapping User names for Interested
@@ -295,7 +359,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                              values may be WithProperties instances.
 
         @type useTls: boolean
-        @param useTls: Send emails using TLS and authenticate with the 
+        @param useTls: Send emails using TLS and authenticate with the
                        smtp host. Defaults to False.
 
         @type smtpUser: string
@@ -326,7 +390,7 @@ class MailNotifier(base.StatusReceiverMultiService):
             for r in extraRecipients:
                 if not isinstance(r, str) or not VALID_EMAIL.search(r):
                     config.error(
-                            "extra recipient %r is not a valid email" % (r,))
+                        "extra recipient %r is not a valid email" % (r,))
         self.extraRecipients = extraRecipients
         self.sendToInterestedUsers = sendToInterestedUsers
         self.fromaddr = fromaddr
@@ -339,8 +403,12 @@ class MailNotifier(base.StatusReceiverMultiService):
                 mode = (mode,)
         for m in mode:
             if m not in self.possible_modes:
-                config.error(
-                    "mode %s is not a valid mode" % (m,))
+                if m == "all":
+                    config.error(
+                        "mode 'all' is not valid in an iterator and must be passed in as a separate string")
+                else:
+                    config.error(
+                        "mode %s is not a valid mode" % (m,))
         self.mode = mode
         self.categories = categories
         self.builders = builders
@@ -351,7 +419,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                 'Newlines are not allowed in email subjects')
         self.subject = subject
         if lookup is not None:
-            if type(lookup) is str:
+            if isinstance(lookup, str):
                 lookup = Domain(lookup)
             assert interfaces.IEmailLookup.providedBy(lookup)
         self.lookup = lookup
@@ -373,7 +441,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         self.master_status = None
 
         # you should either limit on builders or categories, not both
-        if self.builders != None and self.categories != None:
+        if self.builders is not None and self.categories is not None:
             config.error(
                 "Please specify only builders or categories to include - " +
                 "not both.")
@@ -393,16 +461,13 @@ class MailNotifier(base.StatusReceiverMultiService):
 
     def startService(self):
         if self.buildSetSummary:
-            self.buildSetSubscription = \
-            self.master.subscribeToBuildsetCompletions(self.buildsetFinished)
- 
+            self.summarySubscribe()
+
         base.StatusReceiverMultiService.startService(self)
 
     def stopService(self):
-        if self.buildSetSubscription is not None:
-            self.buildSetSubscription.unsubscribe()
-            self.buildSetSubscription = None
-            
+        self.summaryUnsubscribe()
+
         return base.StatusReceiverMultiService.stopService(self)
 
     def disownServiceParent(self):
@@ -414,11 +479,11 @@ class MailNotifier(base.StatusReceiverMultiService):
 
     def builderAdded(self, name, builder):
         # only subscribe to builders we are interested in
-        if self.categories != None and builder.category not in self.categories:
+        if self.categories is not None and builder.category not in self.categories:
             return None
 
         self.watched.append(builder)
-        return self # subscribe to this builder
+        return self  # subscribe to this builder
 
     def builderRemoved(self, name):
         pass
@@ -433,10 +498,10 @@ class MailNotifier(base.StatusReceiverMultiService):
         # here is where we actually do something.
         builder = build.getBuilder()
         if self.builders is not None and builder.name not in self.builders:
-            return False # ignore this build
+            return False  # ignore this build
         if self.categories is not None and \
-               builder.category not in self.categories:
-            return False # ignore this build
+                builder.category not in self.categories:
+            return False  # ignore this build
 
         prev = self.getPreviousBuild(build)
         if "change" in self.mode:
@@ -456,9 +521,15 @@ class MailNotifier(base.StatusReceiverMultiService):
 
         return False
 
+    def sendBuildSetSummary(self, buildset, builds):
+        # only include builds for which isMailNeeded returns true
+        builds = [build for build in builds if self.isMailNeeded(build, build.getResults())]
+        if builds:
+            self.buildMessage("(whole buildset)", builds, buildset['results'])
+
     def buildFinished(self, name, build, results):
-        if ( not self.buildSetSummary and
-             self.isMailNeeded(build, results) ):
+        if (not self.buildSetSummary and
+                self.isMailNeeded(build, results)):
             # for testing purposes, buildMessage returns a Deferred that fires
             # when the mail has been sent. To help unit tests, we return that
             # Deferred here even though the normal IStatusReceiver.buildFinished
@@ -467,40 +538,6 @@ class MailNotifier(base.StatusReceiverMultiService):
             # rearrange this.
             return self.buildMessage(name, [build], results)
         return None
-    
-    def _gotBuilds(self, res, buildset):
-        builds = []
-        for (builddictlist, builder) in res:
-            for builddict in builddictlist:
-                build = builder.getBuild(builddict['number'])
-                if build is not None and self.isMailNeeded(build, build.results):
-                    builds.append(build)
-
-        if builds:
-            self.buildMessage("Buildset Complete: " + buildset['reason'], builds,
-                              buildset['results'])
-        
-    def _gotBuildRequests(self, breqs, buildset):
-        dl = []
-        for breq in breqs:
-            buildername = breq['buildername']
-            builder = self.master_status.getBuilder(buildername)
-            d = self.master.db.builds.getBuildsForRequest(breq['brid'])
-            d.addCallback(lambda builddictlist, builder=builder:
-                          (builddictlist, builder))
-            dl.append(d)
-        d = defer.gatherResults(dl)
-        d.addCallback(self._gotBuilds, buildset)
-
-    def _gotBuildSet(self, buildset, bsid):
-        d = self.master.db.buildrequests.getBuildRequests(bsid=bsid)
-        d.addCallback(self._gotBuildRequests, buildset)
-        
-    def buildsetFinished(self, bsid, result):
-        d = self.master.db.buildsets.getBuildset(bsid=bsid)
-        d.addCallback(self._gotBuildSet, bsid)
-            
-        return d
 
     def getCustomMesgData(self, mode, name, build, results, master_status):
         #
@@ -529,7 +566,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                  'buildText': build.getText(),
                  'buildProperties': build.getProperties(),
                  'slavename': build.getSlavename(),
-                 'reason':  build.getReason().replace('\n', ''),
+                 'reason': build.getReason().replace('\n', ''),
                  'responsibleUsers': build.getResponsibleUsers(),
                  'branch': "",
                  'revision': "",
@@ -567,7 +604,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         # we'll attach it as MIMEText; otherwise, it gets attached as a binary
         # file.  This will suit the vast majority of cases, since utf8 is by
         # far the most common encoding.
-        if type(patch[1]) != types.UnicodeType:
+        if not isinstance(patch[1], types.UnicodeType):
             try:
                     unicode = patch[1].decode('utf8')
             except UnicodeDecodeError:
@@ -582,7 +619,7 @@ class MailNotifier(base.StatusReceiverMultiService):
             a = MIMENonMultipart('application', 'octet-stream')
             a.set_payload(patch[1])
         a.add_header('Content-Disposition', "attachment",
-                    filename="source patch " + str(index) )
+                     filename="source patch " + str(index))
         return a
 
     def createEmail(self, msgdict, builderName, title, results, builds=None,
@@ -592,11 +629,11 @@ class MailNotifier(base.StatusReceiverMultiService):
         if 'subject' in msgdict:
             subject = msgdict['subject'].encode(ENCODING)
         else:
-            subject = self.subject % { 'result': Results[results],
-                                       'projectName': title,
-                                       'title': title,
-                                       'builder': builderName,
-                                       }
+            subject = self.subject % {'result': Results[results],
+                                      'projectName': title,
+                                      'title': title,
+                                      'builder': builderName,
+                                      }
 
         assert '\n' not in subject, \
             "Subject cannot contain newlines"
@@ -625,8 +662,16 @@ class MailNotifier(base.StatusReceiverMultiService):
             for log in logs:
                 name = "%s.%s" % (log.getStep().getName(),
                                   log.getName())
-                if ( self._shouldAttachLog(log.getName()) or
-                     self._shouldAttachLog(name) ):
+                if (self._shouldAttachLog(log.getName()) or
+                        self._shouldAttachLog(name)):
+                    # Use distinct filenames for the e-mail summary
+                    if self.buildSetSummary:
+                        filename = "%s.%s.%s" % (log.getStep().getBuild().getBuilder().getName(),
+                                                 log.getStep().getName(),
+                                                 log.getName())
+                    else:
+                        filename = name
+
                     text = log.getText()
                     if not isinstance(text, unicode):
                         # guess at the encoding, and use replacement symbols
@@ -635,7 +680,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                     a = MIMEText(text.encode(ENCODING),
                                  _charset=ENCODING)
                     a.add_header('Content-Disposition', "attachment",
-                                 filename=name)
+                                 filename=filename)
                     m.attach(a)
 
         #@todo: is there a better way to do this?
@@ -646,39 +691,39 @@ class MailNotifier(base.StatusReceiverMultiService):
                 d = builds[0].render(self.extraHeaders)
             else:
                 d = defer.succeed(self.extraHeaders)
+
             @d.addCallback
             def addExtraHeaders(extraHeaders):
-                for k,v in extraHeaders.items():
+                for k, v in extraHeaders.items():
                     if k in m:
                         twlog.msg("Warning: Got header " + k +
-                          " in self.extraHeaders "
-                          "but it already exists in the Message - "
-                          "not adding it.")
+                                  " in self.extraHeaders "
+                                  "but it already exists in the Message - "
+                                  "not adding it.")
                     m[k] = v
             d.addCallback(lambda _: m)
             return d
-    
+
         return defer.succeed(m)
-    
+
     def buildMessageDict(self, name, build, results):
         if self.customMesg:
             # the customMesg stuff can be *huge*, so we prefer not to load it
             attrs = self.getCustomMesgData(self.mode, name, build, results,
                                            self.master_status)
             text, type = self.customMesg(attrs)
-            msgdict = { 'body' : text, 'type' : type }
+            msgdict = {'body': text, 'type': type}
         else:
             msgdict = self.messageFormatter(self.mode, name, build, results,
                                             self.master_status)
-        
-        return msgdict
 
+        return msgdict
 
     def buildMessage(self, name, builds, results):
         patches = []
         logs = []
-        msgdict = {"body":""}
-        
+        msgdict = {"body": ""}
+
         for build in builds:
             ss_list = build.getSourceStamps()
             if self.addPatch:
@@ -687,7 +732,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                         patches.append(ss.patch)
             if self.addLogs:
                 logs.extend(build.getLogs())
-            
+
             tmp = self.buildMessageDict(name=build.getBuilder().name,
                                         build=build, results=build.results)
             msgdict['body'] += tmp['body']
@@ -727,7 +772,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         return users.getBuildContacts(self.master, build, ['email'])
 
     def _shouldAttachLog(self, logname):
-        if type(self.addLogs) is bool:
+        if isinstance(self.addLogs, bool):
             return self.addLogs
         return logname in self.addLogs
 
@@ -736,7 +781,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         cc_recipients = set()
 
         for r in reduce(list.__add__, rlist, []):
-            if r is None: # getAddress didn't like this address
+            if r is None:  # getAddress didn't like this address
                 continue
 
             # Git can give emails like 'User' <user@foo.com>@foo.com so check
@@ -747,7 +792,7 @@ class MailNotifier(base.StatusReceiverMultiService):
             if VALID_EMAIL.search(r):
                 to_recipients.add(r)
             else:
-                twlog.msg("INVALID EMAIL: %r" + r)
+                twlog.msg("INVALID EMAIL: %r" % r)
 
         # If we're sending to interested users put the extras in the
         # CC list so they can tell if they are also interested in the
@@ -765,7 +810,7 @@ class MailNotifier(base.StatusReceiverMultiService):
 
     def sendmail(self, s, recipients):
         result = defer.Deferred()
-        
+
         if have_ssl and self.useTls:
             client_factory = ssl.ClientContextFactory()
             client_factory.method = SSLv3_METHOD
@@ -776,7 +821,7 @@ class MailNotifier(base.StatusReceiverMultiService):
             useAuth = True
         else:
             useAuth = False
-        
+
         if not ESMTPSenderFactory:
             raise RuntimeError("twisted-mail is not installed - cannot "
                                "send mail")
@@ -788,11 +833,10 @@ class MailNotifier(base.StatusReceiverMultiService):
             requireAuthentication=useAuth)
 
         reactor.connectTCP(self.relayhost, self.smtpPort, sender_factory)
-        
+
         return result
 
     def sendMessage(self, m, recipients):
         s = m.as_string()
         twlog.msg("sending mail (%d bytes) to" % len(s), recipients)
         return self.sendmail(s, recipients)
-
