@@ -14,23 +14,36 @@
 # Copyright Buildbot Team Members
 
 
-from twisted.web import html
-import urllib, time
-from twisted.python import log
-from twisted.internet import defer
+import collections
+import time
+import urllib
+
 from buildbot import interfaces
-from buildbot.status.web.base import HtmlResource, BuildLineMixin, \
-    path_to_build, path_to_slave, path_to_builder, path_to_change, \
-    path_to_root, ICurrentBox, build_get_class, \
-    map_branches, path_to_authzfail, ActionResource, \
-    getRequestCharset
+from buildbot import util
 from buildbot.schedulers.forcesched import ForceScheduler
 from buildbot.schedulers.forcesched import ValidationError
-from buildbot.status.web.build import BuildsResource, StatusResourceBuild
-from buildbot import util
-import collections
+from buildbot.status.web.base import ActionResource
+from buildbot.status.web.base import BuildLineMixin
+from buildbot.status.web.base import HtmlResource
+from buildbot.status.web.base import ICurrentBox
+from buildbot.status.web.base import build_get_class
+from buildbot.status.web.base import getRequestCharset
+from buildbot.status.web.base import map_branches
+from buildbot.status.web.base import path_to_authzfail
+from buildbot.status.web.base import path_to_build
+from buildbot.status.web.base import path_to_builder
+from buildbot.status.web.base import path_to_change
+from buildbot.status.web.base import path_to_root
+from buildbot.status.web.base import path_to_slave
+from buildbot.status.web.build import BuildsResource
+from buildbot.status.web.build import StatusResourceBuild
+from twisted.internet import defer
+from twisted.python import log
+from twisted.web import html
+
 
 class ForceAction(ActionResource):
+
     @defer.inlineCallbacks
     def force(self, req, builderNames):
         master = self.getBuildmaster(req)
@@ -45,12 +58,12 @@ class ForceAction(ActionResource):
         # decode all of the args
         encoding = getRequestCharset(req)
         for name, argl in req.args.iteritems():
-           if name == "checkbox":
-               # damn html's ungeneric checkbox implementation...
-               for cb in argl:
-                   args[cb.decode(encoding)] = True
-           else:
-               args[name] = [ arg.decode(encoding) for arg in argl ]
+            if name == "checkbox":
+                # damn html's ungeneric checkbox implementation...
+                for cb in argl:
+                    args[cb.decode(encoding)] = True
+            else:
+                args[name] = [arg.decode(encoding) for arg in argl]
 
         for sch in master.allSchedulers():
             if schedulername == sch.name:
@@ -58,8 +71,10 @@ class ForceAction(ActionResource):
                     yield sch.force(owner, builderNames, **args)
                     msg = ""
                 except ValidationError, e:
-                    msg = html.escape(e.message.encode('ascii','ignore'))
+                    msg = html.escape(e.message.encode('ascii', 'ignore'))
                 break
+        else:
+            msg = 'no scheduler found'
 
         # send the user back to the builder page
         defer.returnValue(msg)
@@ -91,6 +106,7 @@ class ForceAllBuildsActionResource(ForceAction):
         # back to the welcome page
         defer.returnValue((path_to_root(req) + "builders", msg))
 
+
 class StopAllBuildsActionResource(ActionResource):
 
     def __init__(self, status, selectedOrAll):
@@ -127,6 +143,43 @@ class StopAllBuildsActionResource(ActionResource):
         # go back to the welcome page
         defer.returnValue(path_to_root(req))
 
+
+class CancelAllPendingBuildsActionResource(ActionResource):
+
+    def __init__(self, status, selectedOrAll):
+        self.status = status
+        self.selectedOrAll = selectedOrAll
+        self.action = 'cancelAllPendingBuilds'
+
+    @defer.inlineCallbacks
+    def performAction(self, req):
+        authz = self.getAuthz(req)
+        res = yield authz.actionAllowed('cancelAllPendingBuilds', req)
+        if not res:
+            defer.returnValue(path_to_authzfail(req))
+            return
+
+        builders = None
+        if self.selectedOrAll == 'all':
+            builders = self.status.getBuilderNames()
+        elif self.selectedOrAll == 'selected':
+            builders = [b for b in req.args.get("selected", []) if b]
+
+        c = interfaces.IControl(self.getBuildmaster(req))
+        for bname in builders:
+            authz = self.getAuthz(req)
+            builder_control = c.getBuilder(bname)
+
+            brcontrols = yield builder_control.getPendingBuildRequestControls()
+
+            for build_req in brcontrols:
+                log.msg("Cancelling %s" % build_req)
+                build_req.cancel()
+
+        # go back to the welcome page
+        defer.returnValue(path_to_root(req))
+
+
 class PingBuilderActionResource(ActionResource):
 
     def __init__(self, builder_status):
@@ -137,7 +190,7 @@ class PingBuilderActionResource(ActionResource):
     def performAction(self, req):
         log.msg("web ping of builder '%s'" % self.builder_status.getName())
         res = yield self.getAuthz(req).actionAllowed('pingBuilder', req,
-                                                    self.builder_status)
+                                                     self.builder_status)
         if not res:
             log.msg("..but not authorized")
             defer.returnValue(path_to_authzfail(req))
@@ -149,6 +202,7 @@ class PingBuilderActionResource(ActionResource):
         # send the user back to the builder page
         defer.returnValue(path_to_builder(req, self.builder_status))
 
+
 class ForceBuildActionResource(ForceAction):
 
     def __init__(self, builder_status):
@@ -159,7 +213,7 @@ class ForceBuildActionResource(ForceAction):
     def performAction(self, req):
         # check if this is allowed
         res = yield self.getAuthz(req).actionAllowed(self.action, req,
-                                             self.builder_status)
+                                                     self.builder_status)
         if not res:
             log.msg("..but not authorized")
             defer.returnValue(path_to_authzfail(req))
@@ -172,30 +226,30 @@ class ForceBuildActionResource(ForceAction):
         # send the user back to the builder page
         defer.returnValue((path_to_builder(req, self.builder_status), msg))
 
+
 def buildForceContextForField(req, default_props, sch, field, master, buildername):
-    pname = "%s.%s"%(sch.name, field.fullName)
-    
+    pname = "%s.%s" % (sch.name, field.fullName)
+
     default = field.default
 
     if "list" in field.type:
         choices = field.getChoices(master, sch, buildername)
-        if choices:
-            default = choices[0]
-        default_props[pname+".choices"] = choices
-            
+        default_props[pname + ".choices"] = choices
+
     default = req.args.get(pname, [default])[0]
     if "bool" in field.type:
         default = "checked" if default else ""
     elif isinstance(default, unicode):
         # filter out unicode chars, and html stuff
-        default = html.escape(default.encode('utf-8','ignore'))
+        default = html.escape(default.encode('utf-8', 'ignore'))
 
     default_props[pname] = default
-        
+
     if "nested" in field.type:
         for subfield in field.fields:
             buildForceContextForField(req, default_props, sch, subfield, master, buildername)
 
+
 def buildForceContext(cxt, req, master, buildername=None):
     force_schedulers = {}
     default_props = collections.defaultdict(str)
@@ -204,11 +258,13 @@ def buildForceContext(cxt, req, master, buildername=None):
             force_schedulers[sch.name] = sch
             for field in sch.all_fields:
                 buildForceContextForField(req, default_props, sch, field, master, buildername)
-                
+
     cxt['force_schedulers'] = force_schedulers
     cxt['default_props'] = default_props
 
 # /builders/$builder
+
+
 class StatusResourceBuilder(HtmlResource, BuildLineMixin):
     addSlash = True
 
@@ -230,7 +286,7 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
         if when is not None:
             b['when'] = util.formatInterval(when)
             b['when_time'] = time.strftime("%H:%M:%S",
-                                      time.localtime(time.time() + when))
+                                           time.localtime(time.time() + when))
 
         step = build.getCurrentStep()
         # TODO: is this necessarily the case?
@@ -250,13 +306,30 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
     def content(self, req, cxt):
         b = self.builder_status
 
+        # Grab all the parameters which are prefixed with 'property.'.
+        # We'll use these to filter the builds and build requests we
+        # show below.
+        props = {}
+        prop_prefix = 'property.'
+        for arg, val in req.args.iteritems():
+            if arg.startswith(prop_prefix):
+                props[arg[len(prop_prefix):]] = val[0]
+
+        def prop_match(oprops):
+            for key, val in props.iteritems():
+                if key not in oprops or val != str(oprops[key]):
+                    return False
+            return True
+
         cxt['name'] = b.getName()
         cxt['description'] = b.getDescription()
         req.setHeader('Cache-Control', 'no-cache')
         slaves = b.getSlaves()
         connected_slaves = [s for s in slaves if s.isConnected()]
 
-        cxt['current'] = [self.builder(x, req) for x in b.getCurrentBuilds()]
+        cxt['current'] = [
+            self.builder(x, req) for x in b.getCurrentBuilds()
+            if prop_match(x.getProperties())]
 
         cxt['pending'] = []
         statuses = yield b.getPendingBuildRequestStatuses()
@@ -268,28 +341,34 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
             bsid = yield pb.getBsid()
 
             properties = yield \
-                    pb.master.db.buildsets.getBuildsetProperties(bsid)
+                pb.master.db.buildsets.getBuildsetProperties(bsid)
+            if not prop_match(properties):
+                continue
 
             if source.changes:
                 for c in source.changes:
-                    changes.append({ 'url' : path_to_change(req, c),
-                                     'who' : c.who,
-                                     'revision' : c.revision,
-                                     'repo' : c.repository })
+                    changes.append({'url': path_to_change(req, c),
+                                    'who': c.who,
+                                    'revision': c.revision,
+                                    'repo': c.repository})
 
             cxt['pending'].append({
                 'when': time.strftime("%b %d %H:%M:%S",
                                       time.localtime(submitTime)),
                 'delay': util.formatInterval(util.now() - submitTime),
                 'id': pb.brid,
-                'changes' : changes,
-                'num_changes' : len(changes),
-                'properties' : properties,
-                })
+                'changes': changes,
+                'num_changes': len(changes),
+                'properties': properties,
+            })
 
         numbuilds = cxt['numbuilds'] = int(req.args.get('numbuilds', [self.numbuilds])[0])
+        maxsearch = int(req.args.get('maxsearch', [200])[0])
         recent = cxt['recent'] = []
-        for build in b.generateFinishedBuilds(num_builds=int(numbuilds)):
+        for build in b.generateFinishedBuilds(
+                num_builds=int(numbuilds),
+                max_search=maxsearch,
+                filter_fn=lambda b: prop_match(b.getProperties())):
             recent.append(self.get_line_values(req, build, False))
 
         sl = cxt['slaves'] = []
@@ -301,7 +380,7 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
             s['name'] = slave.getName()
             c = s['connected'] = slave.isConnected()
             s['paused'] = slave.isPaused()
-            s['admin'] = unicode(slave.getAdmin() or '', 'utf-8')
+            s['admin'] = slave.getAdmin() or u''
             if c:
                 connected_slaves += 1
         cxt['connected_slaves'] = connected_slaves
@@ -329,6 +408,7 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
 
         return HtmlResource.getChild(self, path, req)
 
+
 class CancelChangeResource(ActionResource):
 
     def __init__(self, builder_status):
@@ -358,7 +438,7 @@ class CancelChangeResource(ActionResource):
                 if cancel_all or (build_req.brid == request_id):
                     log.msg("Cancelling %s" % build_req)
                     res = yield authz.actionAllowed('cancelPendingBuild', req,
-                                                                build_req)
+                                                    build_req)
                     if res:
                         build_req.cancel()
                     else:
@@ -369,6 +449,7 @@ class CancelChangeResource(ActionResource):
 
         defer.returnValue(path_to_builder(req, self.builder_status))
 
+
 class StopChangeMixin(object):
 
     @defer.inlineCallbacks
@@ -388,7 +469,7 @@ class StopChangeMixin(object):
             build_controls = dict((x.brid, x) for x in brcontrols)
 
             build_req_statuses = yield \
-                    builder_status.getPendingBuildRequestStatuses()
+                builder_status.getPendingBuildRequestStatuses()
 
             for build_req in build_req_statuses:
                 ss = yield build_req.getSourceStamp()
@@ -466,6 +547,8 @@ class StatusResourceAllBuilders(HtmlResource, BuildLineMixin):
             return self.stopall(req)
         if path == "stopchangeall":
             return StopChangeAllResource(self.status)
+        if path == "cancelpendingall":
+            return CancelAllPendingBuildsActionResource(self.status, 'all')
 
         return HtmlResource.getChild(self, path, req)
 
@@ -476,6 +559,8 @@ class StatusResourceAllBuilders(HtmlResource, BuildLineMixin):
         return StopAllBuildsActionResource(self.status, 'all')
 
 # /builders/_selected
+
+
 class StatusResourceSelectedBuilders(HtmlResource, BuildLineMixin):
 
     def __init__(self, status):
@@ -487,6 +572,8 @@ class StatusResourceSelectedBuilders(HtmlResource, BuildLineMixin):
             return self.forceselected(req)
         if path == "stopselected":
             return self.stopselected(req)
+        if path == "cancelpendingselected":
+            return CancelAllPendingBuildsActionResource(self.status, 'selected')
 
         return HtmlResource.getChild(self, path, req)
 
@@ -497,6 +584,8 @@ class StatusResourceSelectedBuilders(HtmlResource, BuildLineMixin):
         return StopAllBuildsActionResource(self.status, 'selected')
 
 # /builders
+
+
 class BuildersResource(HtmlResource):
     pageTitle = "Builders"
     addSlash = True
@@ -511,13 +600,14 @@ class BuildersResource(HtmlResource):
         encoding = getRequestCharset(req)
 
         builders = req.args.get("builder", status.getBuilderNames())
-        branches = [ b.decode(encoding)
-                for b in req.args.get("branch", [])
-                if b ]
+        branches = [b.decode(encoding)
+                    for b in req.args.get("branch", [])
+                    if b]
 
         # get counts of pending builds for each builder
         brstatus_ds = []
         brcounts = {}
+
         def keep_count(statuses, builderName):
             brcounts[builderName] = len(statuses)
         for builderName in builders:
@@ -534,8 +624,8 @@ class BuildersResource(HtmlResource):
         online = 0
         base_builders_url = path_to_root(req) + "builders/"
         for bn in builders:
-            bld = { 'link': base_builders_url + urllib.quote(bn, safe=''),
-                    'name': bn }
+            bld = {'link': base_builders_url + urllib.quote(bn, safe=''),
+                   'name': bn}
             bs.append(bld)
 
             builder = status.getBuilder(bn)
@@ -584,4 +674,3 @@ class BuildersResource(HtmlResource):
             return StatusResourceSelectedBuilders(self.getStatus(req))
 
         return HtmlResource.getChild(self, path, req)
-
