@@ -13,19 +13,23 @@
 #
 # Copyright Buildbot Team Members
 
-import time
 import operator
 import re
+import time
 import urllib
-from twisted.internet import defer
+
 from buildbot import util
+from buildbot.changes import changes
 from buildbot.status import builder
 from buildbot.status.web.base import HtmlResource
-from buildbot.changes import changes
+from twisted.internet import defer
 
-class DoesNotPassFilter(Exception): pass # Used for filtering revs
 
-def getResultsClass(results, prevResults, inProgress):
+class DoesNotPassFilter(Exception):
+    pass  # Used for filtering revs
+
+
+def getResultsClass(results, prevResults, inProgress, inBuilder):
     """Given the current and past results, return the class that will be used
     by the css to display the right color for a box."""
 
@@ -33,7 +37,10 @@ def getResultsClass(results, prevResults, inProgress):
         return "running"
 
     if results is None:
-        return "notstarted"
+        if inBuilder:
+            return "notstarted"
+        else:
+            return "notinbuilder"
 
     if results == builder.SUCCESS:
         return "success"
@@ -53,16 +60,21 @@ def getResultsClass(results, prevResults, inProgress):
         else:
             # The previous build also failed.
             return "failure-again"
-  
+
     # Any other results? Like EXCEPTION?
     return "exception"
 
-class ANYBRANCH: pass # a flag value, used below
+
+class ANYBRANCH:
+    pass  # a flag value, used below
+
 
 class DevRevision:
+
     """Helper class that contains all the information we need for a revision."""
 
     def __init__(self, change):
+        self.codebase = change.codebase
         self.revision = change.revision
         self.comments = change.comments
         self.who = change.who
@@ -74,22 +86,22 @@ class DevRevision:
 
 
 class DevBuild:
+
     """Helper class that contains all the information we need for a build."""
 
-    def __init__(self, revision, build, details):
-        self.revision = revision
-        self.results =  build.getResults()
+    def __init__(self, build, details):
+        self.results = build.getResults()
         self.number = build.getNumber()
         self.isFinished = build.isFinished()
         self.text = build.getText()
         self.eta = build.getETA()
         self.details = details
         self.when = build.getTimes()[0]
-        #TODO: support multiple sourcestamps
-        self.source = build.getSourceStamps()[0]
+        self.sourceStamps = build.getSourceStamps()
 
 
 class ConsoleStatusResource(HtmlResource):
+
     """Main console class. It displays a user-oriented status page.
     Every change is a line in the page, and it shows the result of the first
     build with this change for each slave."""
@@ -115,9 +127,9 @@ class ConsoleStatusResource(HtmlResource):
     def getChangeManager(self, request):
         return request.site.buildbot_service.parent.change_svc
 
-    ##
-    ## Data gathering functions
-    ##
+    #
+    # Data gathering functions
+    #
 
     def getHeadBuild(self, builder):
         """Get the most recent build for the given builder.
@@ -134,22 +146,22 @@ class ConsoleStatusResource(HtmlResource):
     def fetchChangesFromHistory(self, status, max_depth, max_builds, debugInfo):
         """Look at the history of the builders and try to fetch as many changes
         as possible. We need this when the main source does not contain enough
-        sourcestamps. 
+        sourcestamps.
 
         max_depth defines how many builds we will parse for a given builder.
         max_builds defines how many builds total we want to parse. This is to
             limit the amount of time we spend in this function.
-        
+
         This function is sub-optimal, but the information returned by this
         function is cached, so this function won't be called more than once.
         """
-        
+
         allChanges = list()
         build_count = 0
         for builderName in status.getBuilderNames()[:]:
             if build_count > max_builds:
                 break
-            
+
             builder = status.getBuilder(builderName)
             build = self.getHeadBuild(builder)
             depth = 0
@@ -161,18 +173,18 @@ class ConsoleStatusResource(HtmlResource):
                 build = build.getPreviousBuild()
 
         debugInfo["source_fetch_len"] = len(allChanges)
-        return allChanges                
+        return allChanges
 
     @defer.inlineCallbacks
-    def getAllChanges(self, request, status, debugInfo):
+    def getAllChanges(self, request, status, numChanges, debugInfo):
         master = request.site.buildbot_service.master
 
-        chdicts = yield master.db.changes.getRecentChanges(25)
+        chdicts = yield master.db.changes.getRecentChanges(numChanges)
 
         # convert those to Change instances
         allChanges = yield defer.gatherResults([
-                changes.Change.fromChdict(master, chdict)
-                for chdict in chdicts ])
+            changes.Change.fromChdict(master, chdict)
+            for chdict in chdicts])
 
         allChanges.sort(key=self.comparator.getSortingKey())
 
@@ -193,7 +205,7 @@ class ConsoleStatusResource(HtmlResource):
         details = {}
         if not build.getLogs():
             return details
-        
+
         for step in build.getSteps():
             (result, reason) = step.getResults()
             if result == builder.FAILURE:
@@ -202,7 +214,7 @@ class ConsoleStatusResource(HtmlResource):
                 # Remove html tags from the error text.
                 stripHtml = re.compile(r'<.*?>')
                 strippedDetails = stripHtml.sub('', ' '.join(step.getText()))
-                
+
                 details['buildername'] = builderName
                 details['status'] = strippedDetails
                 details['reason'] = reason
@@ -212,7 +224,7 @@ class ConsoleStatusResource(HtmlResource):
                     for log in step.getLogs():
                         logname = log.getName()
                         logurl = request.childLink(
-                          "../builders/%s/builds/%s/steps/%s/logs/%s" % 
+                            "../builders/%s/builds/%s/steps/%s/logs/%s" %
                             (urllib.quote(builderName),
                              build.getNumber(),
                              urllib.quote(name),
@@ -227,70 +239,31 @@ class ConsoleStatusResource(HtmlResource):
         build, and we go down until we find a build that was built prior to the
         last change we are interested in."""
 
-        revision = lastRevision 
-
         builds = []
         build = self.getHeadBuild(builder)
         number = 0
         while build and number < numBuilds:
             debugInfo["builds_scanned"] += 1
 
-            got_rev = None
-            sourceStamps = build.getSourceStamps(absolute=True)
-
-            # The console page cannot handle builds that have more than 1 revision
-            if codebase is not None:
-                # Get the last revision in this build for this codebase.
-                for ss in sourceStamps:
-                    if ss.codebase == codebase:
-                        got_rev = ss.revision
-                        break
-            elif len(sourceStamps) == 1:
-                ss = sourceStamps[0]
-                # Get the last revision in this build.
-                got_rev = ss.revision
-                    
-            # We ignore all builds that don't have last revisions.
-            # TODO(nsylvain): If the build is over, maybe it was a problem
-            # with the update source step. We need to find a way to tell the
-            # user that his change might have broken the source update.
-            if got_rev is not None:
-                number += 1
+            # With multiple codebases cannot determine the last required build,
+            # so take them all except forced builds.
+            if len(build.getChanges()):
                 details = self.getBuildDetails(request, builderName, build)
-                devBuild = DevBuild(got_rev, build, details)
+                devBuild = DevBuild(build, details)
                 builds.append(devBuild)
-
-                # Now break if we have enough builds.
-                current_revision = self.getChangeForBuild(
-                    build, revision)
-                if self.comparator.isRevisionEarlier(
-                    devBuild, current_revision):
-                    break
+                number += 1
 
             build = build.getPreviousBuild()
 
         return builds
 
-    def getChangeForBuild(self, build, revision):
-        if not build or not build.getChanges(): # Forced build
-            return DevBuild(revision, build, None)
-        
-        for change in build.getChanges():
-            if change.revision == revision:
-                return change
-
-        # No matching change, return the last change in build.
-        changes = list(build.getChanges())
-        changes.sort(key=self.comparator.getSortingKey())
-        return changes[-1]
-    
     def getAllBuildsForRevision(self, status, request, codebase, lastRevision,
                                 numBuilds, categories, builders, debugInfo):
         """Returns a dictionary of builds we need to inspect to be able to
         display the console page. The key is the builder name, and the value is
         an array of build we care about. We also returns a dictionary of
         builders we care about. The key is it's category.
- 
+
         codebase is the codebase to get revisions from
         lastRevision is the last revision we want to display in the page.
         categories is a list of categories to display. It is coming from the
@@ -340,11 +313,9 @@ class ConsoleStatusResource(HtmlResource):
 
         return (builderList, allBuilds)
 
-
-    ##
-    ## Display functions
-    ##
-
+    #
+    # Display functions
+    #
     def displayCategories(self, builderList, debugInfo):
         """Display the top category line."""
 
@@ -352,12 +323,11 @@ class ConsoleStatusResource(HtmlResource):
         for category in builderList:
             count += len(builderList[category])
 
-        categories = builderList.keys()
-        categories.sort()
-        
+        categories = sorted(builderList.keys())
+
         cs = []
-        
-        for category in categories:            
+
+        for category in categories:
             c = {}
 
             c["name"] = category
@@ -365,9 +335,9 @@ class ConsoleStatusResource(HtmlResource):
             # To be able to align the table correctly, we need to know
             # what percentage of space this category will be taking. This is
             # (#Builders in Category) / (#Builders Total) * 100.
-            c["size"] = (len(builderList[category]) * 100) / count            
+            c["size"] = (len(builderList[category]) * 100) / count
             cs.append(c)
-            
+
         return cs
 
     def displaySlaveLine(self, status, builderList, debugInfo):
@@ -381,8 +351,7 @@ class ConsoleStatusResource(HtmlResource):
             nbSlaves += len(builderList[category])
 
         # Get the categories, and order them alphabetically.
-        categories = builderList.keys()
-        categories.sort()
+        categories = sorted(builderList.keys())
 
         slaves = {}
 
@@ -391,30 +360,43 @@ class ConsoleStatusResource(HtmlResource):
             slaves[category] = []
             # For each builder in this category, we set the build info and we
             # display the box.
-            for builder in builderList[category]:
+            for bldr in builderList[category]:
                 s = {}
                 s["color"] = "notstarted"
-                s["pageTitle"] = builder
-                s["url"] = "./builders/%s" % urllib.quote(builder)
-                state, builds = status.getBuilder(builder).getState()
+                s["pageTitle"] = bldr
+                s["url"] = "./builders/%s" % urllib.quote(bldr)
+                state, builds = status.getBuilder(bldr).getState()
                 # Check if it's offline, if so, the box is purple.
                 if state == "offline":
                     s["color"] = "offline"
                 else:
                     # If not offline, then display the result of the last
                     # finished build.
-                    build = self.getHeadBuild(status.getBuilder(builder))
+                    build = self.getHeadBuild(status.getBuilder(bldr))
                     while build and not build.isFinished():
                         build = build.getPreviousBuild()
 
                     if build:
                         s["color"] = getResultsClass(build.getResults(), None,
-                                                      False)
+                                                     False, True)
 
                 slaves[category].append(s)
 
         return slaves
 
+    def isCodebaseInBuild(self, build, codebase):
+        """Check if codebase is used in build"""
+        return any(ss.codebase == codebase for ss in build.sourceStamps)
+
+    def isRevisionInBuild(self, build, revision):
+        """ Check if revision is in changes in build """
+        for ss in build.sourceStamps:
+            if ss.codebase == revision.codebase:
+                for change in ss.changes:
+                    if change.revision == revision.revision:
+                        return True
+        return False
+
     def displayStatusLine(self, builderList, allBuilds, revision, debugInfo):
         """Display the boxes that represent the status of each builder in the
         first build "revision" was in. Returns an HTML list of errors that
@@ -426,29 +408,32 @@ class ConsoleStatusResource(HtmlResource):
             nbSlaves += len(builderList[category])
 
         # Sort the categories.
-        categories = builderList.keys()
-        categories.sort()
-        
+        categories = sorted(builderList.keys())
+
         builds = {}
-  
+
         # Display the boxes by category group.
         for category in categories:
-  
+
             builds[category] = []
-            
+
             # Display the boxes for each builder in this category.
-            for builder in builderList[category]:
+            for bldr in builderList[category]:
                 introducedIn = None
                 firstNotIn = None
+                # If there is no builds default to True
+                inBuilder = len(allBuilds[bldr]) == 0
 
-                # Find the first build that does not include the revision.
-                for build in allBuilds[builder]:
-                    if self.comparator.isRevisionEarlier(build, revision):
+                # Find the first build that include the revision.
+                for build in allBuilds[bldr]:
+                    if introducedIn:
                         firstNotIn = build
                         break
-                    else:
-                        introducedIn = build
-                        
+                    elif self.isCodebaseInBuild(build, revision.codebase):
+                        inBuilder = True
+                        if self.isRevisionInBuild(build, revision):
+                            introducedIn = build
+
                 # Get the results of the first build with the revision, and the
                 # first build that does not include the revision.
                 results = None
@@ -463,17 +448,17 @@ class ConsoleStatusResource(HtmlResource):
                     isRunning = True
 
                 url = "./waterfall"
-                pageTitle = builder
+                pageTitle = bldr
                 tag = ""
                 current_details = {}
                 if introducedIn:
                     current_details = introducedIn.details or ""
-                    url = "./buildstatus?builder=%s&number=%s" % (urllib.quote(builder),
-                                                                  introducedIn.number)
+                    url = "./buildstatus?builder=%s&amp;number=%s" % (urllib.quote(bldr),
+                                                                      introducedIn.number)
                     pageTitle += " "
                     pageTitle += urllib.quote(' '.join(introducedIn.text), ' \n\\/:')
 
-                    builderStrip = builder.replace(' ', '')
+                    builderStrip = bldr.replace(' ', '')
                     builderStrip = builderStrip.replace('(', '')
                     builderStrip = builderStrip.replace(')', '')
                     builderStrip = builderStrip.replace('.', '')
@@ -481,10 +466,10 @@ class ConsoleStatusResource(HtmlResource):
 
                 if isRunning:
                     pageTitle += ' ETA: %ds' % (introducedIn.eta or 0)
-                    
-                resultsClass = getResultsClass(results, previousResults, isRunning)
 
-                b = {}                
+                resultsClass = getResultsClass(results, previousResults, isRunning, inBuilder)
+
+                b = {}
                 b["url"] = url
                 b["pageTitle"] = pageTitle
                 b["color"] = resultsClass
@@ -508,7 +493,7 @@ class ConsoleStatusResource(HtmlResource):
                 for rev in reversed(revisions):
                     yield DevRevision(rev)
             else:
-                for index,rev in enumerate(reversed(revisions)):
+                for index, rev in enumerate(reversed(revisions)):
                     if index >= max_revs:
                         break
                     yield DevRevision(rev)
@@ -517,7 +502,7 @@ class ConsoleStatusResource(HtmlResource):
                 if max_revs and index >= max_revs:
                     break
                 try:
-                    for field,acceptable in filter.iteritems():
+                    for field, acceptable in filter.iteritems():
                         if not hasattr(rev, field):
                             raise DoesNotPassFilter
                         if type(acceptable) in (str, unicode):
@@ -558,10 +543,10 @@ class ConsoleStatusResource(HtmlResource):
         # For each revision we show one line
         for revision in revisions:
             r = {}
-            
+
             # Fill the dictionary with this new information
             r['id'] = revision.revision
-            r['link'] = revision.revlink 
+            r['link'] = revision.revlink
             r['who'] = revision.who
             r['date'] = revision.date
             r['comments'] = revision.comments
@@ -570,14 +555,14 @@ class ConsoleStatusResource(HtmlResource):
 
             # Display the status for all builders.
             (builds, details) = self.displayStatusLine(builderList,
-                                            allBuilds,
-                                            revision,
-                                            debugInfo)
+                                                       allBuilds,
+                                                       revision,
+                                                       debugInfo)
             r['builds'] = builds
             r['details'] = details
 
             # Calculate the td span for the comment and the details.
-            r["span"] = len(builderList) + 2            
+            r["span"] = len(builderList) + 2
 
             subs['revisions'].append(r)
 
@@ -587,7 +572,6 @@ class ConsoleStatusResource(HtmlResource):
         debugInfo["load_time"] = time.time() - debugInfo["load_time"]
         return subs
 
-
     def content(self, request, cxt):
         "This method builds the main console view display."
 
@@ -608,7 +592,7 @@ class ConsoleStatusResource(HtmlResource):
         if not reload_time:
             reload_time = 60
 
-        # Append the tag to refresh the page. 
+        # Append the tag to refresh the page.
         if reload_time is not None and reload_time != 0:
             cxt['refresh'] = reload_time
 
@@ -637,16 +621,12 @@ class ConsoleStatusResource(HtmlResource):
 
         # Keep only the revisions we care about.
         # By default we process the last 40 revisions.
-        # If a dev name is passed, we look for the changes by this person in the
-        # last 80 revisions.
         numRevs = int(request.args.get("revs", [40])[0])
-        if devName:
-            numRevs *= 2
-        numBuilds = numRevs
 
         # Get all changes we can find.  This is a DB operation, so it must use
         # a deferred.
-        d = self.getAllChanges(request, status, debugInfo)
+        d = self.getAllChanges(request, status, numRevs, debugInfo)
+
         def got_changes(allChanges):
             debugInfo["source_all"] = len(allChanges)
 
@@ -662,7 +642,7 @@ class ConsoleStatusResource(HtmlResource):
             if codebase is not None:
                 revFilter['codebase'] = codebase
             revisions = list(self.filterRevisions(allChanges, max_revs=numRevs,
-                                                            filter=revFilter))
+                                                  filter=revFilter))
             debugInfo["revision_final"] = len(revisions)
 
             # Fetch all the builds for all builders until we get the next build
@@ -674,13 +654,13 @@ class ConsoleStatusResource(HtmlResource):
                 debugInfo["last_revision"] = lastRevision
 
                 (builderList, allBuilds) = self.getAllBuildsForRevision(status,
-                                                    request,
-                                                    codebase,
-                                                    lastRevision,
-                                                    numBuilds,
-                                                    categories,
-                                                    builders,
-                                                    debugInfo)
+                                                                        request,
+                                                                        codebase,
+                                                                        lastRevision,
+                                                                        numRevs,
+                                                                        categories,
+                                                                        builders,
+                                                                        debugInfo)
 
             debugInfo["added_blocks"] = 0
 
@@ -696,14 +676,16 @@ class ConsoleStatusResource(HtmlResource):
         d.addCallback(got_changes)
         return d
 
+
 class RevisionComparator(object):
+
     """Used for comparing between revisions, as some
     VCS use a plain counter for revisions (like SVN)
     while others use different concepts (see Git).
     """
-    
+
     # TODO (avivby): Should this be a zope interface?
-    
+
     def isRevisionEarlier(self, first_change, second_change):
         """Used for comparing 2 changes"""
         raise NotImplementedError
@@ -714,18 +696,22 @@ class RevisionComparator(object):
 
     def getSortingKey(self):
         raise NotImplementedError
-    
+
+
 class TimeRevisionComparator(RevisionComparator):
+
     def isRevisionEarlier(self, first, second):
         return first.when < second.when
 
     def isValidRevision(self, revision):
-        return True # No general way of determining
+        return True  # No general way of determining
 
     def getSortingKey(self):
         return operator.attrgetter('when')
 
+
 class IntegerRevisionComparator(RevisionComparator):
+
     def isRevisionEarlier(self, first, second):
         try:
             return int(first.revision) < int(second.revision)
@@ -741,4 +727,3 @@ class IntegerRevisionComparator(RevisionComparator):
 
     def getSortingKey(self):
         return operator.attrgetter('revision')
-
