@@ -14,15 +14,22 @@
 # Copyright Buildbot Team Members
 
 
-import time, urllib
+import time
+import urllib
+
+from twisted.internet import defer
 from twisted.web import html
+from twisted.web.resource import NoResource
 from twisted.web.util import Redirect
-from twisted.web.resource import NoResource
-from twisted.internet import defer
 
-from buildbot.status.web.base import HtmlResource, abbreviate_age, \
-    BuildLineMixin, ActionResource, path_to_slave, path_to_authzfail
 from buildbot import util
+from buildbot.status.web.base import ActionResource
+from buildbot.status.web.base import BuildLineMixin
+from buildbot.status.web.base import HtmlResource
+from buildbot.status.web.base import abbreviate_age
+from buildbot.status.web.base import path_to_authzfail
+from buildbot.status.web.base import path_to_slave
+
 
 class ShutdownActionResource(ActionResource):
 
@@ -33,8 +40,8 @@ class ShutdownActionResource(ActionResource):
     @defer.inlineCallbacks
     def performAction(self, request):
         res = yield self.getAuthz(request).actionAllowed(self.action,
-                                                        request,
-                                                        self.slave)
+                                                         request,
+                                                         self.slave)
 
         url = None
         if res:
@@ -44,6 +51,7 @@ class ShutdownActionResource(ActionResource):
             url = path_to_authzfail(request)
         defer.returnValue(url)
 
+
 class PauseActionResource(ActionResource):
 
     def __init__(self, slave, state):
@@ -54,8 +62,8 @@ class PauseActionResource(ActionResource):
     @defer.inlineCallbacks
     def performAction(self, request):
         res = yield self.getAuthz(request).actionAllowed(self.action,
-                                                        request,
-                                                        self.slave)
+                                                         request,
+                                                         self.slave)
 
         url = None
         if res:
@@ -66,8 +74,11 @@ class PauseActionResource(ActionResource):
         defer.returnValue(url)
 
 # /buildslaves/$slavename
+
+
 class OneBuildSlaveResource(HtmlResource, BuildLineMixin):
     addSlash = False
+
     def __init__(self, slavename):
         HtmlResource.__init__(self)
         self.slavename = slavename
@@ -84,10 +95,10 @@ class OneBuildSlaveResource(HtmlResource, BuildLineMixin):
             return PauseActionResource(slave, path == "pause")
         return Redirect(path_to_slave(req, slave))
 
-    def content(self, request, ctx):        
+    def content(self, request, ctx):
         s = self.getStatus(request)
         slave = s.getSlave(self.slavename)
-        
+
         my_builders = []
         for bname in s.getBuilderNames():
             b = s.getBuilder(bname)
@@ -99,15 +110,15 @@ class OneBuildSlaveResource(HtmlResource, BuildLineMixin):
         current_builds = []
         for b in my_builders:
             for cb in b.getCurrentBuilds():
-                if cb.getSlavename() == self.slavename:                    
+                if cb.getSlavename() == self.slavename:
                     current_builds.append(self.get_line_values(request, cb))
 
         try:
             max_builds = int(request.args.get('numbuilds')[0])
         except:
             max_builds = 10
-           
-        recent_builds = []    
+
+        recent_builds = []
         n = 0
         for rb in s.generateFinishedBuilds(builders=[b.getName() for b in my_builders]):
             if rb.getSlavename() == self.slavename:
@@ -126,24 +137,27 @@ class OneBuildSlaveResource(HtmlResource, BuildLineMixin):
             pause_url = request.childLink("pause")
 
         ctx.update(dict(slave=slave,
-                        slavename = self.slavename,  
-                        current = current_builds, 
-                        recent = recent_builds, 
-                        shutdown_url = request.childLink("shutdown"),
-                        pause_url = pause_url,
-                        authz = self.getAuthz(request),
-                        this_url = "../../../" + path_to_slave(request, slave),
-                        access_uri = slave.getAccessURI()),
-                        admin = unicode(slave.getAdmin() or '', 'utf-8'),
-                        host = unicode(slave.getHost() or '', 'utf-8'),
-                        slave_version = slave.getVersion(),
-                        show_builder_column = True,
-                        connect_count = connect_count)
+                        slavename=self.slavename,
+                        current=current_builds,
+                        recent=recent_builds,
+                        shutdown_url=request.childLink("shutdown"),
+                        pause_url=pause_url,
+                        authz=self.getAuthz(request),
+                        this_url="../../../" + path_to_slave(request, slave),
+                        access_uri=slave.getAccessURI(),
+                        admin=slave.getAdmin() or u'',
+                        host=slave.getHost() or u'',
+                        info=slave.getInfoAsDict(),
+                        slave_version=slave.getVersion(),
+                        show_builder_column=True,
+                        connect_count=connect_count))
         template = request.site.buildbot_service.templates.get_template("buildslave.html")
         data = template.render(**ctx)
         return data
 
 # /buildslaves
+
+
 class BuildSlavesResource(HtmlResource):
     pageTitle = "BuildSlaves"
     addSlash = True
@@ -152,7 +166,7 @@ class BuildSlavesResource(HtmlResource):
         s = self.getStatus(request)
 
         #?no_builders=1 disables build column
-        show_builder_column = not (request.args.get('no_builders', '0')[0])=='1'
+        show_builder_column = not (request.args.get('no_builders', '0')[0]) == '1'
         ctx['show_builder_column'] = show_builder_column
 
         used_by_builder = {}
@@ -171,25 +185,25 @@ class BuildSlavesResource(HtmlResource):
             slave = s.getSlave(name)
             slave_status = s.botmaster.slaves[name].slave_status
             info['running_builds'] = len(slave_status.getRunningBuilds())
-            info['link'] = request.childLink(urllib.quote(name,''))
+            info['link'] = request.childLink(urllib.quote(name, ''))
             info['name'] = name
 
             if show_builder_column:
                 info['builders'] = []
                 for b in used_by_builder.get(name, []):
                     info['builders'].append(dict(link=request.childLink("../builders/%s" % b), name=b))
-                                        
+
             info['version'] = slave.getVersion()
             info['connected'] = slave.isConnected()
             info['connectCount'] = slave.getConnectCount()
             info['paused'] = slave.isPaused()
-            
-            info['admin'] = unicode(slave.getAdmin() or '', 'utf-8')
+
+            info['admin'] = slave.getAdmin() or u''
             last = slave.lastMessageReceived()
             if last:
                 info['last_heard_from_age'] = abbreviate_age(time.time() - last)
                 info['last_heard_from_time'] = time.strftime("%Y-%b-%d %H:%M:%S",
-                                                            time.localtime(last))
+                                                             time.localtime(last))
 
         template = request.site.buildbot_service.templates.get_template("buildslaves.html")
         data = template.render(**ctx)
