@@ -19,15 +19,19 @@
 import datetime
 import os
 import re
+import urllib
 
 from twisted.internet import defer
-from twisted.web import html, resource, server
+from twisted.web import html
+from twisted.web import resource
+from twisted.web import server
 
 from buildbot.status.web.base import HtmlResource
+from buildbot.status.web.base import path_to_root
 from buildbot.util import json
 
 
-_IS_INT = re.compile('^[-+]?\d+$')
+_IS_INT = re.compile(r'^[-+]?\d+$')
 
 
 FLAGS = """\
@@ -121,6 +125,7 @@ def FilterOut(data):
 
 
 class JsonResource(resource.Resource):
+
     """Base class for json data."""
 
     contentType = "application/json"
@@ -147,7 +152,7 @@ class JsonResource(resource.Resource):
                                 pageTitle=pageTitle,
                                 parent_node=self)
         # Equivalent to resource.Resource.getChildWithDefault()
-        if self.children.has_key(path):
+        if path in self.children:
             return self.children[path]
         return self.getChild(path, request)
 
@@ -164,7 +169,8 @@ class JsonResource(resource.Resource):
 
     def render_GET(self, request):
         """Renders a HTTP GET at the http request level."""
-        d = defer.maybeDeferred(lambda : self.content(request))
+        d = defer.maybeDeferred(lambda: self.content(request))
+
         def handle(data):
             if isinstance(data, unicode):
                 data = data.encode("utf-8")
@@ -174,22 +180,24 @@ class JsonResource(resource.Resource):
             else:
                 request.setHeader("content-type", self.contentType)
                 request.setHeader("content-disposition",
-                                "attachment; filename=\"%s.json\"" % request.path)
+                                  "attachment; filename=\"%s.json\"" % request.path)
             # Make sure we get fresh pages.
             if self.cache_seconds:
                 now = datetime.datetime.utcnow()
                 expires = now + datetime.timedelta(seconds=self.cache_seconds)
                 request.setHeader("Expires",
-                                expires.strftime("%a, %d %b %Y %H:%M:%S GMT"))
+                                  expires.strftime("%a, %d %b %Y %H:%M:%S GMT"))
                 request.setHeader("Pragma", "no-cache")
             return data
         d.addCallback(handle)
+
         def ok(data):
             request.write(data)
             request.finish()
+
         def fail(f):
             request.processingFailed(f)
-            return None # processingFailed will log this for us
+            return None  # processingFailed will log this for us
         d.addCallbacks(ok, fail)
         return server.NOT_DONE_YET
 
@@ -210,7 +218,7 @@ class JsonResource(resource.Resource):
             data = {}
             # Remove superfluous /
             select = [s.strip('/') for s in select]
-            select.sort(cmp=lambda x,y: cmp(x.count('/'), y.count('/')),
+            select.sort(cmp=lambda x, y: cmp(x.count('/'), y.count('/')),
                         reverse=True)
             for item in select:
                 # Start back at root.
@@ -222,7 +230,11 @@ class JsonResource(resource.Resource):
                 postpath = request.postpath[:]
                 request.postpath = filter(None, item.split('/'))
                 while request.postpath and not child.isLeaf:
-                    pathElement = request.postpath.pop(0)
+                    # Twisted unquotes the querystring once. We unquote once more
+                    # to allow for "doubly escaped" elements, which makes it possible
+                    # to select on resource names with slashes in them without them being
+                    # split into separate (invalid) elements.
+                    pathElement = urllib.unquote(request.postpath.pop(0))
                     node[pathElement] = {}
                     node = node[pathElement]
                     request.prepath.append(pathElement)
@@ -231,29 +243,29 @@ class JsonResource(resource.Resource):
                 # some asDict methods return a Deferred, so handle that
                 # properly
                 if hasattr(child, 'asDict'):
-                    child_dict = yield defer.maybeDeferred(lambda :
-                                                child.asDict(request))
+                    child_dict = yield defer.maybeDeferred(lambda:
+                                                           child.asDict(request))
                 else:
                     child_dict = {
-                        'error' : 'Not available',
+                        'error': 'Not available',
                     }
                 node.update(child_dict)
 
                 request.prepath = prepath
                 request.postpath = postpath
         else:
-            data = yield defer.maybeDeferred(lambda : self.asDict(request))
+            data = yield defer.maybeDeferred(lambda: self.asDict(request))
 
         if filter_out:
             data = FilterOut(data)
         if compact:
-            data = json.dumps(data, sort_keys=True, separators=(',',':'))
+            data = json.dumps(data, sort_keys=True, separators=(',', ':'))
         else:
             data = json.dumps(data, sort_keys=True, indent=2)
         if callback:
             # Only accept things that look like identifiers for now
             callback = callback[0]
-            if re.match(r'^[a-zA-Z$][a-zA-Z$0-9.]*$', callback):
+            if re.match(r'^[a-zA-Z$_][a-zA-Z$0-9._]*$', callback):
                 data = '%s(%s);' % (callback, data)
         defer.returnValue(data)
 
@@ -267,8 +279,8 @@ class JsonResource(resource.Resource):
             for name in self.children:
                 child = self.getChildWithDefault(name, request)
                 if isinstance(child, JsonResource):
-                    data[name] = yield defer.maybeDeferred(lambda :
-                                            child.asDict(request))
+                    data[name] = yield defer.maybeDeferred(lambda:
+                                                           child.asDict(request))
                 # else silently pass over non-json resources.
             defer.returnValue(data)
         else:
@@ -315,11 +327,11 @@ def ToHtml(text):
             else:
                 line_full = line + '&as_text=1'
             output.append('<a href="' + html.escape(line_full) + '">' +
-                html.escape(line) + '</a>')
+                          html.escape(line) + '</a>')
         else:
             output.append(html.escape(line).replace('  ', '&nbsp;&nbsp;'))
         if not in_item:
-            output.append('<br>')
+            output.append('<br/>')
 
     if in_item:
         output.append('</li>')
@@ -330,6 +342,7 @@ def ToHtml(text):
 
 
 class HelpResource(HtmlResource):
+
     def __init__(self, text, pageTitle, parent_node):
         HtmlResource.__init__(self)
         self.text = text
@@ -340,15 +353,16 @@ class HelpResource(HtmlResource):
     def content(self, request, cxt):
         cxt['level'] = self.parent_level
         cxt['text'] = ToHtml(self.text)
-        cxt['children'] = [ n for n in self.parent_children if n != 'help' ]
+        cxt['children'] = [n for n in self.parent_children if n != 'help']
         cxt['flags'] = ToHtml(FLAGS)
         cxt['examples'] = ToHtml(EXAMPLES).replace(
-                'href="/json',
-                'href="../%sjson' % (self.parent_level * '../'))
+            'href="/json',
+            'href="%s' % path_to_root(request) + 'json')
 
         template = request.site.buildbot_service.templates.get_template("jsonhelp.html")
         return template.render(**cxt)
 
+
 class BuilderPendingBuildsJsonResource(JsonResource):
     help = """Describe pending builds for a builder.
 """
@@ -361,9 +375,10 @@ class BuilderPendingBuildsJsonResource(JsonResource):
     def asDict(self, request):
         # buildbot.status.builder.BuilderStatus
         d = self.builder_status.getPendingBuildRequestStatuses()
+
         def to_dict(statuses):
             return defer.gatherResults(
-                [ b.asDict_async() for b in statuses ])
+                [b.asDict_async() for b in statuses])
         d.addCallback(to_dict)
         return d
 
@@ -380,8 +395,8 @@ class BuilderJsonResource(JsonResource):
         self.putChild('slaves', BuilderSlavesJsonResources(status,
                                                            builder_status))
         self.putChild(
-                'pendingBuilds',
-                BuilderPendingBuildsJsonResource(status, builder_status))
+            'pendingBuilds',
+            BuilderPendingBuildsJsonResource(status, builder_status))
 
     def asDict(self, request):
         # buildbot.status.builder.BuilderStatus
@@ -672,12 +687,13 @@ class SourceStampJsonResource(JsonResource):
         self.putChild('changes',
                       ChangesJsonResource(status, source_stamp.changes))
         # TODO(maruel): Should redirect to the patch's url instead.
-        #if source_stamp.patch:
+        # if source_stamp.patch:
         #  self.putChild('patch', StaticHTML(source_stamp.path))
 
     def asDict(self, request):
         return self.source_stamp.asDict()
 
+
 class MetricsJsonResource(JsonResource):
     help = """Master metrics.
 """
@@ -692,8 +708,8 @@ class MetricsJsonResource(JsonResource):
             return None
 
 
-
 class JsonStatusResource(JsonResource):
+
     """Retrieves all json data."""
     help = """JSON status
 
