@@ -13,23 +13,32 @@
 #
 # Copyright Buildbot Team Members
 
+import re
+import urllib
 import xml.dom.minidom
 import xml.parsers.expat
 
-from twisted.python import log
-from twisted.internet import defer
+from string import lower
+from urlparse import urlparse
+from urlparse import urlunparse
 
+from twisted.internet import defer
+from twisted.internet import reactor
+from twisted.python import log
+
+from buildbot.config import ConfigErrors
+from buildbot.interfaces import BuildSlaveTooOldError
 from buildbot.process import buildstep
 from buildbot.steps.source.base import Source
-from buildbot.interfaces import BuildSlaveTooOldError
-from buildbot.config import ConfigErrors
+
 
 class SVN(Source):
+
     """I perform Subversion checkout/update operations."""
 
     name = 'svn'
 
-    renderables = [ 'repourl' ]
+    renderables = ['repourl']
     possible_modes = ('incremental', 'full')
     possible_methods = ('clean', 'fresh', 'clobber', 'copy', 'export', None)
 
@@ -66,16 +75,29 @@ class SVN(Source):
         self.stdio_log = self.addLogForRemoteCommands("stdio")
 
         d = self.checkSvn()
+
         def checkInstall(svnInstalled):
             if not svnInstalled:
                 raise BuildSlaveTooOldError("SVN is not installed on slave")
             return 0
         d.addCallback(checkInstall)
 
+        d.addCallback(lambda _: self.sourcedirIsPatched())
+
+        def checkPatched(patched):
+            if patched:
+                return self.purge(False)
+            else:
+                return 0
+        d.addCallback(checkPatched)
+
         if self.mode == 'full':
             d.addCallback(self.full)
         elif self.mode == 'incremental':
             d.addCallback(self.incremental)
+
+        if patch:
+            d.addCallback(self.patch, patch)
         d.addCallback(self.parseGotRevision)
         d.addCallback(self.finish)
         d.addErrback(self.failed)
@@ -92,14 +114,8 @@ class SVN(Source):
 
         updatable = yield self._sourcedirIsUpdatable()
         if not updatable:
-            # blow away the old (un-updatable) directory
-            yield self.runRmdir(self.workdir)
-
-            # then do a checkout
-            checkout_cmd = ['checkout', self.repourl, '.']
-            if self.revision:
-                checkout_cmd.extend(["--revision", str(self.revision)])
-            yield self._dovccmd(checkout_cmd)
+            # blow away the old (un-updatable) directory and checkout
+            yield self.clobber()
         elif self.method == 'clean':
             yield self.clean()
         elif self.method == 'fresh':
@@ -110,29 +126,19 @@ class SVN(Source):
         updatable = yield self._sourcedirIsUpdatable()
 
         if not updatable:
-            # blow away the old (un-updatable) directory
-            yield self.runRmdir(self.workdir)
-
-            # and plan to do a checkout
-            command = ['checkout', self.repourl, '.']
+            # blow away the old (un-updatable) directory and checkout
+            yield self.clobber()
         else:
             # otherwise, do an update
             command = ['update']
+            if self.revision:
+                command.extend(['--revision', str(self.revision)])
+            yield self._dovccmd(command)
 
-        if self.revision:
-            command.extend(['--revision', str(self.revision)])
-
-        yield self._dovccmd(command)
-
-    @defer.inlineCallbacks
     def clobber(self):
-        yield self.runRmdir(self.workdir)
-
-        checkout_cmd = ['checkout', self.repourl, '.']
-        if self.revision:
-            checkout_cmd.extend(["--revision", str(self.revision)])
-
-        yield self._dovccmd(checkout_cmd)
+        d = self.runRmdir(self.workdir)
+        d.addCallback(lambda _: self._checkout())
+        return d
 
     def fresh(self):
         d = self.purge(True)
@@ -159,7 +165,7 @@ class SVN(Source):
             old_workdir = self.workdir
             self.workdir = 'source'
             yield self.incremental(None)
-        except: # finally doesn't work in python-2.4
+        except:  # finally doesn't work in python-2.4
             self.workdir = old_workdir
             raise
         self.workdir = old_workdir
@@ -167,16 +173,22 @@ class SVN(Source):
         # if we're copying, copy; otherwise, export from source to build
         if self.method == 'copy':
             cmd = buildstep.RemoteCommand('cpdir',
-                    { 'fromdir': 'source', 'todir':self.workdir,
-                      'logEnviron': self.logEnviron })
+                                          {'fromdir': 'source', 'todir': self.workdir,
+                                           'logEnviron': self.logEnviron})
         else:
             export_cmd = ['svn', 'export']
             if self.revision:
                 export_cmd.extend(["--revision", str(self.revision)])
+            if self.username:
+                export_cmd.extend(['--username', self.username])
+            if self.password is not None:
+                export_cmd.extend(['--password', ('obfuscated', self.password, 'XXXXXX')])
+            if self.extra_args:
+                export_cmd.extend(self.extra_args)
             export_cmd.extend(['source', self.workdir])
 
             cmd = buildstep.RemoteShellCommand('', export_cmd,
-                    env=self.env, logEnviron=self.logEnviron, timeout=self.timeout)
+                                               env=self.env, logEnviron=self.logEnviron, timeout=self.timeout)
         cmd.useLog(self.stdio_log, False)
 
         yield self.runCommand(cmd)
@@ -186,21 +198,21 @@ class SVN(Source):
 
     def finish(self, res):
         d = defer.succeed(res)
+
         def _gotResults(results):
             self.setStatus(self.cmd, results)
             return results
         d.addCallback(_gotResults)
-        d.addCallbacks(self.finished, self.checkDisconnect)
+        d.addCallback(self.finished)
         return d
 
-
-    def _dovccmd(self, command, collectStdout=False):
+    def _dovccmd(self, command, collectStdout=False, abandonOnFailure=True):
         assert command, "No command specified"
         command.extend(['--non-interactive', '--no-auth-cache'])
         if self.username:
             command.extend(['--username', self.username])
-        if self.password:
-            command.extend(['--password', self.password])
+        if self.password is not None:
+            command.extend(['--password', ('obfuscated', self.password, 'XXXXXX')])
         if self.depth:
             command.extend(['--depth', self.depth])
         if self.extra_args:
@@ -212,10 +224,10 @@ class SVN(Source):
                                            timeout=self.timeout,
                                            collectStdout=collectStdout)
         cmd.useLog(self.stdio_log, False)
-        log.msg("Starting SVN command : svn %s" % (" ".join(command), ))
         d = self.runCommand(cmd)
+
         def evaluateCommand(cmd):
-            if cmd.didFail():
+            if cmd.didFail() and abandonOnFailure:
                 log.msg("Source step failed while running command %s" % cmd)
                 raise buildstep.BuildStepFailed()
             if collectStdout:
@@ -251,7 +263,7 @@ class SVN(Source):
             msg = "Corrupted xml, aborting step"
             self.stdio_log.addHeader(msg)
             raise buildstep.BuildStepFailed()
-        defer.returnValue(extractedurl == self.repourl)
+        defer.returnValue(extractedurl == self.svnUriCanonicalize(self.repourl))
         return
 
     @defer.inlineCallbacks
@@ -282,8 +294,8 @@ class SVN(Source):
             try:
                 revision = stdout_xml.getElementsByTagName('commit')[0].attributes['revision'].value
             except (KeyError, IndexError):
-                msg =("SVN.parseGotRevision unable to detect Last Changed Rev in"
-                      " output of svn info")
+                msg = ("SVN.parseGotRevision unable to detect Last Changed Rev in"
+                       " output of svn info")
                 log.msg(msg)
                 # fall through and try to get 'Revision' instead
 
@@ -291,8 +303,8 @@ class SVN(Source):
             try:
                 revision = stdout_xml.getElementsByTagName('entry')[0].attributes['revision'].value
             except (KeyError, IndexError):
-                msg =("SVN.parseGotRevision unable to detect revision in"
-                      " output of svn info")
+                msg = ("SVN.parseGotRevision unable to detect revision in"
+                       " output of svn info")
                 log.msg(msg)
                 raise buildstep.BuildStepFailed()
 
@@ -302,17 +314,17 @@ class SVN(Source):
 
         defer.returnValue(cmd.rc)
 
-
     def purge(self, ignore_ignores):
         """Delete everything that shown up on status."""
         command = ['status', '--xml']
         if ignore_ignores:
             command.append('--no-ignore')
         d = self._dovccmd(command, collectStdout=True)
+
         def parseAndRemove(stdout):
             files = []
             for filename in self.getUnversionedFiles(stdout, self.keep_on_purge):
-                filename = self.workdir+'/'+str(filename)
+                filename = self.workdir + '/' + str(filename)
                 files.append(filename)
             if len(files) == 0:
                 d = defer.succeed(0)
@@ -323,6 +335,7 @@ class SVN(Source):
                     d = self.runRmdir(files, abandonOnFailure=False)
             return d
         d.addCallback(parseAndRemove)
+
         def evaluateCommand(rc):
             if rc != 0:
                 log.msg("Failed removing files")
@@ -366,6 +379,7 @@ class SVN(Source):
                                            timeout=self.timeout)
         cmd.useLog(self.stdio_log, False)
         d = self.runCommand(cmd)
+
         def evaluate(cmd):
             if cmd.rc != 0:
                 return False
@@ -379,3 +393,74 @@ class SVN(Source):
         lastChange = max([int(c.revision) for c in changes])
         return lastChange
 
+    @staticmethod
+    def svnUriCanonicalize(uri):
+        collapse = re.compile(r'([^/]+/\.\./?|/\./|//|/\.$|/\.\.$|^/\.\.)')
+        server_authority = re.compile(r'^(?:([^\@]+)\@)?([^\:]+)(?:\:(.+))?$')
+        default_port = {'http': '80',
+                        'https': '443',
+                        'svn': '3690'}
+
+        relative_schemes = ['http', 'https', 'svn']
+        quote = lambda uri: urllib.quote(uri, "!$&'()*+,-./:=@_~")
+
+        if len(uri) == 0 or uri == '/':
+            return uri
+
+        (scheme, authority, path, parameters, query, fragment) = urlparse(uri)
+        scheme = lower(scheme)
+        if authority:
+            userinfo, host, port = server_authority.match(authority).groups()
+            if host[-1] == '.':
+                host = host[:-1]
+            authority = lower(host)
+            if userinfo:
+                authority = "%s@%s" % (userinfo, authority)
+            if port and port != default_port.get(scheme, None):
+                authority = "%s:%s" % (authority, port)
+
+        if scheme in relative_schemes:
+            last_path = path
+            while True:
+                path = collapse.sub('/', path, 1)
+                if last_path == path:
+                    break
+                last_path = path
+
+        path = quote(urllib.unquote(path))
+        canonical_uri = urlunparse((scheme, authority, path, parameters, query, fragment))
+        if canonical_uri == '/':
+            return canonical_uri
+        elif canonical_uri[-1] == '/' and canonical_uri[-2] != '/':
+            return canonical_uri[:-1]
+        else:
+            return canonical_uri
+
+    def _checkout(self):
+        checkout_cmd = ['checkout', self.repourl, '.']
+        if self.revision:
+            checkout_cmd.extend(["--revision", str(self.revision)])
+        if self.retry:
+            abandonOnFailure = (self.retry[1] <= 0)
+        else:
+            abandonOnFailure = True
+        d = self._dovccmd(checkout_cmd, abandonOnFailure=abandonOnFailure)
+
+        def _retry(res):
+            if self.stopped or res == 0:
+                return res
+            delay, repeats = self.retry
+            if repeats > 0:
+                log.msg("Checkout failed, trying %d more times after %d seconds"
+                        % (repeats, delay))
+                self.retry = (delay, repeats - 1)
+                df = defer.Deferred()
+                df.addCallback(lambda _: self.runRmdir(self.workdir))
+                df.addCallback(lambda _: self._checkout())
+                reactor.callLater(delay, df.callback, None)
+                return df
+            return res
+
+        if self.retry:
+            d.addCallback(_retry)
+        return d
