@@ -13,23 +13,32 @@
 #
 # Copyright Buildbot Team Members
 
+import mock
+import os.path
 import weakref
-from twisted.internet import defer
+
+from buildbot import config
+from buildbot import interfaces
+from buildbot.status import build
 from buildbot.test.fake import fakedb
 from buildbot.test.fake import pbmanager
 from buildbot.test.fake.botmaster import FakeBotMaster
-from buildbot import config
-import mock
+from twisted.internet import defer
+from zope.interface import implements
+
 
 class FakeCache(object):
+
     """Emulate an L{AsyncLRUCache}, but without any real caching.  This
     I{does} do the weakref part, to catch un-weakref-able objects."""
+
     def __init__(self, name, miss_fn):
         self.name = name
         self.miss_fn = miss_fn
 
     def get(self, key, **kwargs):
         d = self.miss_fn(key, **kwargs)
+
         def mkref(x):
             if x is not None:
                 weakref.ref(x)
@@ -37,6 +46,9 @@ class FakeCache(object):
         d.addCallback(mkref)
         return d
 
+    def put(self, key, val):
+        pass
+
 
 class FakeCaches(object):
 
@@ -46,12 +58,31 @@ class FakeCaches(object):
 
 class FakeStatus(object):
 
+    def __init__(self, master):
+        self.master = master
+        self.lastBuilderStatus = None
+
     def builderAdded(self, name, basedir, category=None, description=None):
-        return FakeBuilderStatus()
+        bs = FakeBuilderStatus(self.master)
+        self.lastBuilderStatus = bs
+        return bs
+
+    def slaveConnected(self, name):
+        pass
+
+    def build_started(self, brid, buildername, build_status):
+        pass
 
 
 class FakeBuilderStatus(object):
 
+    implements(interfaces.IBuilderStatus)
+
+    def __init__(self, master):
+        self.master = master
+        self.basedir = os.path.join(master.basedir, 'bldr')
+        self.lastBuildStatus = None
+
     def setDescription(self, description):
         self._description = description
 
@@ -73,8 +104,20 @@ class FakeBuilderStatus(object):
     def setBigState(self, state):
         pass
 
+    def newBuild(self):
+        bld = build.BuildStatus(self, self.master, 3)
+        self.lastBuildStatus = bld
+        return bld
+
+    def buildStarted(self, builderStatus):
+        pass
+
+    def addPointEvent(self, text):
+        pass
+
 
 class FakeMaster(object):
+
     """
     Create a fake Master instance: a Mock with some convenience
     implementations:
@@ -90,7 +133,7 @@ class FakeMaster(object):
         self.basedir = 'basedir'
         self.botmaster = FakeBotMaster(master=self)
         self.botmaster.parent = self
-        self.status = FakeStatus()
+        self.status = FakeStatus(self)
         self.status.master = self
 
     def getObjectId(self):
@@ -99,11 +142,16 @@ class FakeMaster(object):
     def subscribeToBuildRequests(self, callback):
         pass
 
+    def maybeBuildsetComplete(self, bsid):
+        pass
+
     # work around http://code.google.com/p/mock/issues/detail?id=105
     def _get_child_mock(self, **kw):
         return mock.Mock(**kw)
 
 # Leave this alias, in case we want to add more behavior later
+
+
 def make_master(wantDb=False, testcase=None, **kwargs):
     master = FakeMaster(**kwargs)
     if wantDb:
