@@ -13,14 +13,22 @@
 #
 # Copyright Buildbot Team Members
 
-from twisted.trial import unittest
-from twisted.python.reflect import namedModule
-from buildbot.steps.source import bzr
-from buildbot.status.results import SUCCESS, FAILURE
-from buildbot.test.util import sourcesteps
-from buildbot.test.fake.remotecommand import ExpectShell, Expect
 import os.path
 
+from buildbot.status.results import FAILURE
+from buildbot.status.results import RETRY
+from buildbot.status.results import SUCCESS
+from buildbot.steps.source import bzr
+from buildbot.steps.transfer import _FileReader
+from buildbot.test.fake.remotecommand import Expect
+from buildbot.test.fake.remotecommand import ExpectRemoteRef
+from buildbot.test.fake.remotecommand import ExpectShell
+from buildbot.test.util import sourcesteps
+from twisted.internet import error
+from twisted.python.reflect import namedModule
+from twisted.trial import unittest
+
+
 class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
 
     def setUp(self):
@@ -37,6 +45,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -49,9 +60,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -65,6 +76,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file=r'wkdir\.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file=r'wkdir\.bzr',
                                 logEnviron=True))
             + 0,
@@ -77,9 +91,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         return self.runStep()
 
@@ -92,6 +106,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
                         timeout=1,
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -107,9 +124,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
                         timeout=1,
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -123,6 +140,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -135,9 +155,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -150,6 +170,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -162,9 +185,91 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
+        return self.runStep()
+
+    def test_mode_full_clean_patched(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='full', method='clean'))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 0,
+            # clean up the applied patch
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'clean-tree', '--ignored', '--force'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.bzr',
+                                logEnviron=True))
+            + 0,
+            # this clean is from 'mode=clean'
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'clean-tree', '--ignored', '--force'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'update'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                              stdout='100')
+            + 0,
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
+        return self.runStep()
+
+    def test_mode_full_clean_patch(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='full', method='clean'), patch=(1, 'patch'))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
+            Expect('stat', dict(file='wkdir/.bzr',
+                                logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'clean-tree', '--ignored', '--force'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'update'])
+            + 0,
+            Expect('downloadFile', dict(blocksize=16384, maxsize=None,
+                                        reader=ExpectRemoteRef(_FileReader),
+                                        slavedest='.buildbot-diff', workdir='wkdir',
+                                        mode=None))
+            + 0,
+            Expect('downloadFile', dict(blocksize=16384, maxsize=None,
+                                        reader=ExpectRemoteRef(_FileReader),
+                                        slavedest='.buildbot-patched', workdir='wkdir',
+                                        mode=None))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['patch', '-p1', '--remove-empty-files',
+                                 '--force', '--forward', '-i', '.buildbot-diff'])
+            + 0,
+            Expect('rmdir', dict(dir='wkdir/.buildbot-diff',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                              stdout='100')
+            + 0,
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -178,6 +283,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -190,9 +298,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -205,6 +313,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -217,9 +328,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -232,23 +343,67 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('rmdir', dict(dir='wkdir',
                                  logEnviron=True))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
+    def test_mode_full_clobber_retry(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='full', method='clobber', retry=(0, 2)))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+            + 1,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+            + 1,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                              stdout='100')
+            + 0,
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
+        return self.runStep()
 
     def test_mode_full_clobber_revision(self):
         self.setupStep(
@@ -259,25 +414,27 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('rmdir', dict(dir='wkdir',
                                  logEnviron=True))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                'http://bzr.squid-cache.org/bzr/squid3/trunk',
-                                '.', '-r', '3730'])
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk',
+                                 '.', '-r', '3730'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
-
     def test_mode_full_clobber_baseurl(self):
         self.setupStep(
             bzr.Bzr(baseURL='http://bzr.squid-cache.org/bzr/squid3',
@@ -286,24 +443,26 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('rmdir', dict(dir='wkdir',
                                  logEnviron=True))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                os.path.join('http://bzr.squid-cache.org/bzr/squid3', 'trunk'), '.'])
+                                 os.path.join('http://bzr.squid-cache.org/bzr/squid3', 'trunk'), '.'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
-
     def test_mode_full_clobber_baseurl_nodefault(self):
         self.setupStep(
             bzr.Bzr(baseURL='http://bzr.squid-cache.org/bzr/squid3',
@@ -313,24 +472,26 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('rmdir', dict(dir='wkdir',
                                  logEnviron=True))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                os.path.join('http://bzr.squid-cache.org/bzr/squid3', 'branches/SQUID_3_0'), '.'])
+                                 os.path.join('http://bzr.squid-cache.org/bzr/squid3', 'branches/SQUID_3_0'), '.'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-                    )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
-
     def test_mode_full_copy(self):
         self.setupStep(
             bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
@@ -339,6 +500,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('rmdir', dict(dir='build',
                                  logEnviron=True))
             + 0,
@@ -355,9 +519,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='source',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-            )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -370,6 +534,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -379,9 +546,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-            )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -395,6 +562,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 0,
@@ -404,9 +574,9 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100')
+                              stdout='100')
             + 0,
-            )
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
@@ -419,19 +589,57 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='100\n')
+                              stdout='100\n')
             + 0,
-            )
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100\n', 'Bzr')
+        return self.runStep()
+
+    def test_mode_incremental_retry(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='incremental', retry=(0, 1)))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
+            Expect('stat', dict(file='wkdir/.bzr',
+                                logEnviron=True))
+            + 1,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+            + 1,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                              stdout='100\n')
+            + 0,
+        )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
         self.expectProperty('got_revision', '100\n', 'Bzr')
         return self.runStep()
@@ -444,19 +652,22 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'version-info', '--custom', "--template='{revno}"])
             + ExpectShell.log('stdio',
-                stdout='oiasdfj010laksjfd')
+                              stdout='oiasdfj010laksjfd')
             + 0,
-            )
+        )
         self.expectOutcome(result=FAILURE, status_text=["updating"])
         return self.runStep()
 
@@ -468,15 +679,31 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             ExpectShell(workdir='wkdir',
                         command=['bzr', '--version'])
             + 0,
+            Expect('stat', dict(file='wkdir/.buildbot-patched',
+                                logEnviron=True))
+            + 1,
             Expect('stat', dict(file='wkdir/.bzr',
                                 logEnviron=True))
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['bzr', 'checkout',
-                                'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
+                                 'http://bzr.squid-cache.org/bzr/squid3/trunk', '.'])
             + ExpectShell.log('stdio',
-                stderr='failed\n')
+                              stderr='failed\n')
             + 128,
-            )
+        )
         self.expectOutcome(result=FAILURE, status_text=["updating"])
         return self.runStep()
+
+    def test_slave_connection_lost(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='full', method='fresh'))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + ('err', error.ConnectionLost()),
+        )
+        self.expectOutcome(result=RETRY,
+                           status_text=["update", "exception", "slave", "lost"])
+        return self.runStep()
