@@ -15,14 +15,21 @@
 # Portions Copyright 2013 Bad Dog Consulting
 
 
-from twisted.trial import unittest
-from buildbot.steps.source.p4 import P4
+import platform
+import textwrap
+
+from buildbot import config
+from buildbot.status.results import RETRY
 from buildbot.status.results import SUCCESS
+from buildbot.steps.source.p4 import P4
+from buildbot.test.fake.remotecommand import Expect
+from buildbot.test.fake.remotecommand import ExpectShell
 from buildbot.test.util import sourcesteps
 from buildbot.test.util.properties import ConstantRenderable
-from buildbot.test.fake.remotecommand import ExpectShell, Expect
-from buildbot import config
-import textwrap
+from twisted.internet import error
+from twisted.trial import unittest
+
+_is_windows = (platform.system() == 'Windows')
 
 
 class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
@@ -38,7 +45,10 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         self.build.getSourceStamp().revision = args.get('revision', None)
 
         # builddir propety used to create absolute path required in perforce client spec.
-        self.properties.setProperty('builddir', '/home/user/workspace', 'P4')
+        workspace_dir = '/home/user/workspace'
+        if _is_windows:
+            workspace_dir = r'C:\Users\username\Workspace'
+        self.properties.setProperty('builddir', workspace_dir, 'P4')
 
     def test_no_empty_step_config(self):
         self.assertRaises(config.ConfigErrors, lambda: P4())
@@ -80,6 +90,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                           p4user='user', p4client='p4_client1', p4passwd='pass'),
                        dict(revision='100',))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_spec = textwrap.dedent('''\
         Client: p4_client1
 
@@ -88,7 +101,7 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
@@ -96,7 +109,7 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
 
         View:
         \t//depot/trunk/... //p4_client1/...
-        ''');
+        ''' % root_dir)
 
         self.expectCommands(
             ExpectShell(workdir='wkdir',  # defaults to this, only changes if it has a copy mode.
@@ -105,19 +118,19 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
 
             ExpectShell(workdir='wkdir',
                         command=['p4', '-p', 'localhost:12000', '-u', 'user',
-                                       '-P', 'pass', '-c', 'p4_client1',
-                                       'client', '-i'],
+                                 '-P', 'pass', '-c', 'p4_client1',
+                                 'client', '-i'],
                         initialStdin=client_spec)
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['p4', '-p', 'localhost:12000', '-u', 'user',
-                                       '-P', 'pass', '-c', 'p4_client1',
-                                       'sync', '//depot...@100'])
+                                 '-P', 'pass', '-c', 'p4_client1',
+                                 'sync', '//depot...@100'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['p4', '-p', 'localhost:12000', '-u', 'user',
-                                       '-P', 'pass', '-c', 'p4_client1',
-                                       'changes', '-m1', '#have'])
+                                 '-P', 'pass', '-c', 'p4_client1',
+                                 'changes', '-m1', '#have'])
             + ExpectShell.log('stdio',
                               stdout="Change 100 on 2013/03/21 by user@machine \'duh\'")
             + 0,
@@ -126,27 +139,33 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         self.expectProperty('got_revision', '100', 'P4')
         return self.runStep()
 
-    def _incremental(self, client_stdin=''):
+    def _incremental(self, client_stdin='', extra_args=None, workdir='wkdir', timeout=20 * 60):
+        if extra_args is None:
+            extra_args = []
+
         self.expectCommands(
-            ExpectShell(workdir='wkdir',  # defaults to this, only changes if it has a copy mode.
+            ExpectShell(workdir=workdir,
                         command=['p4', '-V'])  # expected remote command
             + 0,  # expected exit status
 
-            ExpectShell(workdir='wkdir',
+            ExpectShell(workdir=workdir,
+                        timeout=timeout,
                         command=['p4', '-p', 'localhost:12000', '-u', 'user',
-                                       '-P', 'pass', '-c', 'p4_client1',
-                                       'client', '-i'],
+                                 '-P', 'pass', '-c', 'p4_client1',
+                                 'client', '-i'],
                         initialStdin=client_stdin,)
             + 0,
-            ExpectShell(workdir='wkdir',
-                        command=['p4', '-p', 'localhost:12000', '-u', 'user',
-                                       '-P', 'pass', '-c', 'p4_client1',
-                                       'sync'])
+            ExpectShell(workdir=workdir,
+                        timeout=timeout,
+                        command=(['p4', '-p', 'localhost:12000', '-u', 'user',
+                                  '-P', 'pass', '-c', 'p4_client1']
+                                 + extra_args + ['sync']))
             + 0,
-            ExpectShell(workdir='wkdir',
+            ExpectShell(workdir=workdir,
+                        timeout=timeout,
                         command=['p4', '-p', 'localhost:12000', '-u', 'user',
-                                       '-P', 'pass', '-c', 'p4_client1',
-                                       'changes', '-m1', '#have'])
+                                 '-P', 'pass', '-c', 'p4_client1',
+                                 'changes', '-m1', '#have'])
             + ExpectShell.log('stdio',
                               stdout="Change 100 on 2013/03/21 by user@machine \'duh\'")
             + 0,
@@ -160,6 +179,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                           p4base='//depot', p4branch='trunk',
                           p4user='user', p4client='p4_client1', p4passwd='pass'))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_spec = textwrap.dedent('''\
         Client: p4_client1
 
@@ -168,7 +190,7 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
@@ -176,7 +198,7 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
 
         View:
         \t//depot/trunk/... //p4_client1/...
-        ''')
+        ''' % root_dir)
         self._incremental(client_stdin=client_spec)
 
     def test_mode_incremental_p4base_with_p4extra_views(self):
@@ -185,6 +207,10 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                           p4extra_views=[('-//depot/trunk/test', 'test'),
                                          ('-//depot/trunk/doc', 'doc')],
                           p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_spec = textwrap.dedent('''\
         Client: p4_client1
 
@@ -193,7 +219,7 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
@@ -203,13 +229,17 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         \t//depot/trunk/... //p4_client1/...
         \t-//depot/trunk/test/... //p4_client1/test/...
         \t-//depot/trunk/doc/... //p4_client1/doc/...
-        ''')
+        ''' % root_dir)
         self._incremental(client_stdin=client_spec)
 
     def test_mode_incremental_p4viewspec(self):
         self.setupStep(P4(p4port='localhost:12000', mode='incremental',
                           p4viewspec=[('//depot/trunk/', '')],
                           p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_spec = textwrap.dedent('''\
         Client: p4_client1
 
@@ -218,7 +248,7 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
@@ -226,38 +256,183 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
 
         View:
         \t//depot/trunk/... //p4_client1/...
-        ''')
+        ''' % root_dir)
         self._incremental(client_stdin=client_spec)
 
-    def _full(self, client_stdin='', p4client='p4_client1', p4user='user'):
+    def test_mode_incremental_p4viewspec_suffix(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='incremental',
+                          p4viewspec_suffix=None,
+                          p4viewspec=[('//depot/trunk/foo.xml', 'bar.xml')],
+                          p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/foo.xml //p4_client1/bar.xml
+        ''' % root_dir)
+        self._incremental(client_stdin=client_spec)
+
+    def test_mode_incremental_p4client_spec_options(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='incremental',
+                          p4base='//depot', p4branch='trunk',
+                          p4client_spec_options='rmdir compress',
+                          p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\trmdir compress
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._incremental(client_stdin=client_spec)
+
+    def test_mode_incremental_parent_workdir(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='incremental',
+                          p4base='//depot', p4branch='trunk',
+                          p4user='user', p4client='p4_client1', p4passwd='pass',
+                          workdir='../another_wkdir'))
+
+        root_dir = '/home/user/another_wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\another_wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._incremental(client_stdin=client_spec, workdir='../another_wkdir')
+
+    def test_mode_incremental_p4extra_args(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='incremental',
+                          p4base='//depot', p4branch='trunk',
+                          p4user='user', p4client='p4_client1', p4passwd='pass',
+                          p4extra_args=['-Zproxyload']))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._incremental(client_stdin=client_spec, extra_args=['-Zproxyload'])
+
+    def test_mode_incremental_timeout(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='incremental',
+                          p4base='//depot', p4branch='trunk',
+                          p4user='user', p4client='p4_client1', p4passwd='pass',
+                          timeout=60 * 60))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._incremental(client_stdin=client_spec, timeout=60 * 60)
+
+    def _full(self, client_stdin='', p4client='p4_client1', p4user='user', workdir='wkdir', extra_args=None):
+        if extra_args is None:
+            extra_args = []
+
         self.expectCommands(
-            ExpectShell(workdir='wkdir',  # defaults to this, only changes if it has a copy mode.
+            ExpectShell(workdir=workdir,
                         command=['p4', '-V'])  # expected remote command
             + 0,  # expected exit status
 
-            ExpectShell(workdir='wkdir',
+            ExpectShell(workdir=workdir,
                         command=['p4', '-p', 'localhost:12000', '-u', p4user,
-                                       '-P', 'pass', '-c', p4client, 'client',
-                                       '-i'],
+                                 '-P', 'pass', '-c', p4client, 'client',
+                                 '-i'],
                         initialStdin=client_stdin)
             + 0,
-            ExpectShell(workdir='wkdir',
+            ExpectShell(workdir=workdir,
                         command=['p4', '-p', 'localhost:12000', '-u', p4user,
-                                       '-P', 'pass', '-c', p4client, 'sync',
-                                       '#none'])
+                                 '-P', 'pass', '-c', p4client]
+                        + extra_args
+                        + ['sync', '#none'])
             + 0,
 
-            Expect('rmdir', {'dir': 'wkdir', 'logEnviron': True})
+            Expect('rmdir', {'dir': workdir, 'logEnviron': True})
             + 0,
 
-            ExpectShell(workdir='wkdir',
+            ExpectShell(workdir=workdir,
                         command=['p4', '-p', 'localhost:12000', '-u', p4user,
-                                       '-P', 'pass', '-c', p4client, 'sync'])
+                                 '-P', 'pass', '-c', p4client]
+                        + extra_args + ['sync'])
             + 0,
-            ExpectShell(workdir='wkdir',
+            ExpectShell(workdir=workdir,
                         command=['p4', '-p', 'localhost:12000', '-u', p4user,
-                                       '-P', 'pass', '-c', p4client, 'changes',
-                                       '-m1', '#have'])
+                                 '-P', 'pass', '-c', p4client, 'changes',
+                                 '-m1', '#have'])
             + ExpectShell.log('stdio',
                               stdout="Change 100 on 2013/03/21 by user@machine \'duh\'")
             + 0,
@@ -272,6 +447,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                mode='full', p4base='//depot', p4branch='trunk',
                p4user='user', p4client='p4_client1', p4passwd='pass'))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_stdin = textwrap.dedent('''\
         Client: p4_client1
 
@@ -280,14 +458,14 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
         LineEnd:\tlocal
 
         View:
-        \t//depot/trunk/... //p4_client1/...\n''')
+        \t//depot/trunk/... //p4_client1/...\n''' % root_dir)
         self._full(client_stdin=client_stdin)
 
     def test_mode_full_p4viewspec(self):
@@ -295,6 +473,10 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
             P4(p4port='localhost:12000',
                mode='full', p4viewspec=[('//depot/main/', '')],
                p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_stdin = textwrap.dedent('''\
         Client: p4_client1
 
@@ -303,14 +485,14 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
         LineEnd:\tlocal
 
         View:
-        \t//depot/main/... //p4_client1/...\n''')
+        \t//depot/main/... //p4_client1/...\n''' % root_dir)
 
         self._full(client_stdin=client_stdin)
 
@@ -322,6 +504,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                p4branch='release/1.0', p4user='user', p4client='p4_client2',
                p4passwd='pass'))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_stdin = textwrap.dedent('''\
         Client: p4_client2
 
@@ -330,14 +515,14 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
         LineEnd:\tlocal
 
         View:
-        \t//depot/release/1.0/... //p4_client2/...\n''')
+        \t//depot/release/1.0/... //p4_client2/...\n''' % root_dir)
 
         self._full(client_stdin=client_stdin, p4client='p4_client2')
 
@@ -349,6 +534,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                p4user='user', p4client=ConstantRenderable('p4_client_render'),
                p4passwd='pass'))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_stdin = textwrap.dedent('''\
         Client: p4_client_render
 
@@ -357,14 +545,14 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
         LineEnd:\tlocal
 
         View:
-        \t//depot/trunk/... //p4_client_render/...\n''')
+        \t//depot/trunk/... //p4_client_render/...\n''' % root_dir)
 
         self._full(client_stdin=client_stdin, p4client='p4_client_render')
 
@@ -376,6 +564,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                p4branch=ConstantRenderable('render_branch'),
                p4user='user', p4client='p4_client1', p4passwd='pass'))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_stdin = textwrap.dedent('''\
         Client: p4_client1
 
@@ -384,14 +575,14 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
         LineEnd:\tlocal
 
         View:
-        \t//depot/render_branch/... //p4_client1/...\n''')
+        \t//depot/render_branch/... //p4_client1/...\n''' % root_dir)
 
         self._full(client_stdin=client_stdin)
 
@@ -403,6 +594,9 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
                p4user='different_user', p4client='p4_client1',
                p4passwd='pass'))
 
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
         client_stdin = textwrap.dedent('''\
         Client: p4_client1
 
@@ -411,13 +605,140 @@ class TestP4(sourcesteps.SourceStepMixin, unittest.TestCase):
         Description:
         \tCreated by different_user
 
-        Root:\t/home/user/workspace/wkdir
+        Root:\t%s
 
         Options:\tallwrite rmdir
 
         LineEnd:\tlocal
 
         View:
-        \t//depot/render_trunk/... //p4_client1/...\n''')
+        \t//depot/render_trunk/... //p4_client1/...\n''' % root_dir)
 
         self._full(client_stdin=client_stdin, p4user='different_user')
+
+    def test_mode_full_p4viewspec_suffix(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='full',
+                          p4viewspec_suffix=None,
+                          p4viewspec=[('//depot/trunk/foo.xml', 'bar.xml')],
+                          p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/foo.xml //p4_client1/bar.xml
+        ''' % root_dir)
+        self._full(client_stdin=client_spec)
+
+    def test_mode_full_p4client_spec_options(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='full',
+                          p4base='//depot', p4branch='trunk',
+                          p4client_spec_options='rmdir compress',
+                          p4user='user', p4client='p4_client1', p4passwd='pass'))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\trmdir compress
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._full(client_stdin=client_spec)
+
+    def test_mode_full_parent_workdir(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='full',
+                          p4base='//depot', p4branch='trunk',
+                          p4user='user', p4client='p4_client1', p4passwd='pass',
+                          workdir='../another_wkdir'))
+
+        root_dir = '/home/user/another_wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\another_wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._full(client_stdin=client_spec, workdir='../another_wkdir')
+
+    def test_mode_full_p4extra_args(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='full',
+                          p4base='//depot', p4branch='trunk',
+                          p4user='user', p4client='p4_client1', p4passwd='pass',
+                          p4extra_args=['-Zproxyload']))
+
+        root_dir = '/home/user/workspace/wkdir'
+        if _is_windows:
+            root_dir = r'C:\Users\username\Workspace\wkdir'
+        client_spec = textwrap.dedent('''\
+        Client: p4_client1
+
+        Owner: user
+
+        Description:
+        \tCreated by user
+
+        Root:\t%s
+
+        Options:\tallwrite rmdir
+
+        LineEnd:\tlocal
+
+        View:
+        \t//depot/trunk/... //p4_client1/...
+        ''' % root_dir)
+        self._full(client_stdin=client_spec, extra_args=['-Zproxyload'])
+
+    def test_slave_connection_lost(self):
+        self.setupStep(P4(p4port='localhost:12000', mode='incremental',
+                          p4base='//depot', p4branch='trunk',
+                          p4user='user', p4client='p4_client1', p4passwd='pass'),
+                       dict(revision='100',))
+
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['p4', '-V'])
+            + ('err', error.ConnectionLost()),
+        )
+        self.expectOutcome(result=RETRY,
+                           status_text=["update", "exception", "slave", "lost"])
+        return self.runStep()
