@@ -13,15 +13,16 @@
 #
 # Copyright Buildbot Team Members
 
-from twisted.trial import unittest
-from buildbot.steps.source import repo
-from buildbot.status.results import SUCCESS, FAILURE
-from buildbot.test.util import sourcesteps
-from buildbot.test.fake.remotecommand import ExpectShell, Expect
-from buildbot.process.properties import Properties
 from .test_changes_gerritchangesource import TestGerritChangeSource
 from buildbot.changes.changes import Change
-import os
+from buildbot.process.properties import Properties
+from buildbot.status.results import FAILURE
+from buildbot.status.results import SUCCESS
+from buildbot.steps.source import repo
+from buildbot.test.fake.remotecommand import Expect
+from buildbot.test.fake.remotecommand import ExpectShell
+from buildbot.test.util import sourcesteps
+from twisted.trial import unittest
 
 
 class RepoURL(unittest.TestCase):
@@ -100,7 +101,7 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
     def expectClobber(self):
         # stat return 1 so we clobber
         self.expectCommands(
-            Expect('stat', dict(file=os.path.join('wkdir', '.repo'),
+            Expect('stat', dict(file='wkdir/.repo',
                                 logEnviron=self.logEnviron))
             + 1,
             Expect('rmdir', dict(dir='wkdir',
@@ -114,19 +115,20 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
     def expectnoClobber(self):
         # stat return 0, so nothing
         self.expectCommands(
-            Expect('stat', dict(file=os.path.join('wkdir', '.repo'),
+            Expect('stat', dict(file='wkdir/.repo',
                                 logEnviron=self.logEnviron))
             + 0,
         )
 
-    def expectRepoSync(self, which_fail=-1, breakatfail=False, syncoptions=["-c"], override_commands=[]):
+    def expectRepoSync(self, which_fail=-1, breakatfail=False, depth=0,
+                       syncoptions=["-c"], override_commands=[]):
         commands = [
             self.ExpectShell(
                 command=[
                     'bash', '-c', self.step._getCleanupCommand()]),
             self.ExpectShell(
                 command=['repo', 'init', '-u', 'git://myrepo.com/manifest.git',
-                                 '-b', 'mb', '-m', 'mf'])
+                         '-b', 'mb', '-m', 'mf', '--depth', str(depth)])
         ] + override_commands + [
             self.ExpectShell(command=['repo', 'sync'] + syncoptions),
             self.ExpectShell(
@@ -144,6 +146,13 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
         self.expectRepoSync()
         return self.myRunStep(status_text=["update"])
 
+    def test_basic_depth(self):
+        """basic first time repo sync"""
+        self.mySetupStep(repoDownloads=None, depth=2)
+        self.expectClobber()
+        self.expectRepoSync(depth=2)
+        return self.myRunStep(status_text=["update"])
+
     def test_update(self):
         """basic second time repo sync"""
         self.mySetupStep()
@@ -175,13 +184,13 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
         self.expectClobber()
         override_commands = [
             Expect(
-                'stat', dict(file=os.path.join('wkdir', 'http://u.rl/test.manifest'),
+                'stat', dict(file='wkdir/http://u.rl/test.manifest',
                              logEnviron=False)),
             self.ExpectShell(logEnviron=False, command=['wget',
                              'http://u.rl/test.manifest',
-                             '-O', 'manifest_override.xml']),
+                                                        '-O', 'manifest_override.xml']),
             self.ExpectShell(
-                logEnviron=False, workdir=os.path.join('wkdir', '.repo'),
+                logEnviron=False, workdir='wkdir/.repo',
                 command=['ln', '-sf', '../manifest_override.xml',
                          'manifest.xml'])
         ]
@@ -198,13 +207,13 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
                          syncAllBranches=True)
         self.expectClobber()
         override_commands = [
-            Expect('stat', dict(file=os.path.join('wkdir', 'test.manifest'),
+            Expect('stat', dict(file='wkdir/test.manifest',
                                 logEnviron=False)),
             self.ExpectShell(logEnviron=False,
                              command=[
                                  'cp', '-f', 'test.manifest', 'manifest_override.xml']),
             self.ExpectShell(logEnviron=False,
-                             workdir=os.path.join('wkdir', '.repo'),
+                             workdir='wkdir/.repo',
                              command=['ln', '-sf', '../manifest_override.xml',
                                       'manifest.xml'])
         ]
@@ -238,7 +247,7 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
             self.ExpectShell(
                 command=['tar', '-z', '-xvf', '/tarball.tgz']) + 1,
             self.ExpectShell(command=['rm', '-f', '/tarball.tgz']) + 1,
-            Expect('rmdir', dict(dir=os.path.join('wkdir', '.repo'),
+            Expect('rmdir', dict(dir='wkdir/.repo',
                                  logEnviron=False))
             + 1)
         self.expectRepoSync()
@@ -301,7 +310,7 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
             self.ExpectShell(
                 command=['rm', '-f', '/tarball.tar']) + 0,
             Expect(
-                'rmdir', dict(dir=os.path.join('wkdir', '.repo'),
+                'rmdir', dict(dir='wkdir/.repo',
                               logEnviron=False))
             + 0)
         self.expectRepoSync()
@@ -338,9 +347,9 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
                                              ['-cvf', '/tarball.' + suffix, '.repo'])
                             + 1,
                             self.ExpectShell(
-                            command=['rm', '-f', '/tarball.tar']) + 0,
+                                command=['rm', '-f', '/tarball.tar']) + 0,
                             self.ExpectShell(
-                            command=['repo', 'download', 'test/bla', '564/12'])
+                                command=['repo', 'download', 'test/bla', '564/12'])
                             + 0)
         return self.myRunStep(status_text=["update"])
 
@@ -394,16 +403,16 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
             self.ExpectShell(
                 command=['repo', 'init', '-u', 'git://myrepo.com/manifest.git',
-                                 '-b', 'mb', '-m', 'mf'])
+                         '-b', 'mb', '-m', 'mf', '--depth', '0'])
             + 0,
             self.ExpectShell(
-                workdir=os.path.join('wkdir', '.repo', 'manifests'),
+                workdir='wkdir/.repo/manifests',
                 command=[
                     'git', 'fetch', 'git://myrepo.com/manifest.git',
                     'refs/changes/65/565/12'])
             + 0,
             self.ExpectShell(
-                workdir=os.path.join('wkdir', '.repo', 'manifests'),
+                workdir='wkdir/.repo/manifests',
                 command=['git', 'cherry-pick', 'FETCH_HEAD'])
             + 0,
             self.ExpectShell(command=['repo', 'sync', '-c'])
@@ -476,7 +485,7 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
                 command=['repo', 'download', 'test/bla', '564/12'])
             + 1 + Expect.log("stdio", stderr="patch \n"),
             self.ExpectShell(
-                command=['repo', 'forall', '-c',  'git', 'diff', 'HEAD'])
+                command=['repo', 'forall', '-c', 'git', 'diff', 'HEAD'])
             + 0
         )
         return self.myRunStep(result=FAILURE, status_text=["download failed: test/bla 564/12"])
@@ -494,7 +503,7 @@ class TestRepo(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0 +
             Expect.log("stdio", stderr="Automatic cherry-pick failed \n"),
             self.ExpectShell(
-                command=['repo', 'forall', '-c',  'git', 'diff', 'HEAD'])
+                command=['repo', 'forall', '-c', 'git', 'diff', 'HEAD'])
             + 0
         )
         return self.myRunStep(result=FAILURE, status_text=["download failed: test/bla 564/12"])
